/* $Header: /plx/source/Xclients/generic/RCS/hpSpecific.c,v 0.114 1996/06/12 22:34:46 lovvik Exp $ */
/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: hpSpecific.c,v $
 * Revision 0.114  1996/06/12  22:34:46  lovvik
 *    Moved all funtions into video.c  There is no longer a file for
 *    platform-specific functions.
 *
 * Revision 0.113  1996/05/28  22:09:53  lovvik
 *   Moved all functions to utils.c and video.c
 *   These functions are commented out.  Only the
 *   Audio-specific information remains.
 *
 * Revision 0.112  1996/05/20  18:27:11  lovvik
 *   Added a function called OutputOff().  Also, fixed a problem
 *   in ShowInputWorkProc.  A call to XtVaGetValues() did not
 *   pass in the address of the variable, so it caused a bus
 *   error.
 *
 * Revision 0.111  1996/04/21  23:31:34  lovvik
 *   Worked on multiple board support.
 *
 * Revision 0.110  1996/03/08  17:54:44  lovvik
 *   Changed the order of 2 include files to fix a
 *   compilation problem.
 *
 * Revision 0.109  1996/01/25  04:18:18  drapeau
 * Added support for the PlxOutputCascade to automatically create the
 * output menu hierarchy, for enhanced flexibility.
 *
 * Revision 0.108  1996/01/05  23:36:16  drapeau
 * Minor addition to ShowCompactMovieDialog() to account for new
 * interface feature that should be greyed out when this function is
 * called.
 *
 * Revision 0.107  1995/10/06  23:01:15  drapeau
 * Added support for showing the current input signal type in use.
 *
 * Revision 0.106  1995/10/02  21:54:27  drapeau
 * Added support for keeping track of the user interface's visibility
 * state: whether some panels are mapped or not, etc.
 *
 * Revision 0.105  1995/08/23  01:57:18  drapeau
 * Fixed ShowInputWorkProc() so that it correctly resets the recording
 * frame rate based on the input video signal type it detects (30 fps for
 * NTSC, 25 FPS for PAL or SECAM).
 *
 * Revision 0.104  1995/06/23  19:17:36  drapeau
 * Two fixes:
 *
 * 1) Modified CreateJPEGWidget() so that it correctly displays JPEG
 *    images after zooming from one zoom factor to another.  The previous
 *    version would not resize the JPEG image according to the new zoom
 *    factor.
 *
 * 2) Modified ShowInputWorkProc() so that input selection behaves as
 *    with VideoTool.  That is, when an input is explicitly selected,
 *    autoSearch is turned off (since the supposed intent is that the
 *    user wants to show whatever video was selected whether sync was
 *    available or not).
 *
 * Revision 0.103  1995/05/16  02:57:37  drapeau
 * Added support for the InputCascade code written by Dave Arnold, which
 * takes a plx_io data structure and creates a cascading menu-and-button
 * interface that reflects the hardware's input selections.
 *
 * Revision 0.102  1995/05/12  00:15:33  lovvik
 * Modified the initialization of audioMap[].  The structure
 * has been changed to add a string that will serve as a label
 * for each audio object.
 *
 * Revision 0.101  1995/05/02  17:50:12  drapeau
 * Added support for hardware zoom when available.
 *
 * Revision 0.100  1995/04/27  00:28:48  lovvik
 * Added audioMap.  This structure contains:
 *   1)  Command line parameter
 *   2)  Help message
 *   3)  Address of an audio object
 * for each audio object that is supported on HP.
 *
 * Revision 0.99  1994/10/17  21:09:31  drapeau
 * Two changes:
 *
 * 1) In CreateJPEGWidget(), store the previous values of Q factor for
 *    each JPEG widget so they can be restored when the new versions of
 *    the widgets are re-created.
 *
 * 2) In OutputWorkProc(), force the live video window to release its
 *    Port by destroying the VideoIn widget (thereby destroying its
 *    window).
 *
 * Revision 0.98  1994/10/15  01:46:08  drapeau
 * Added code to complete MovieTool's notion of what operations should be
 * allowed under what conditions.  For example, input selection should
 * not be allowed during recording and playback.
 *
 * Revision 0.97  1994/10/14  02:53:00  drapeau
 * Added code to insure that JPEG playback widgets have the correct Q
 * table loaded and that they refresh correctly when output is enabled.
 *
 * Revision 0.96  1994/10/10  23:01:42  drapeau
 * Added functions to allow any of the available input types to be
 * selected.  Also, removed some functions to reflect the interface
 * change (the new function ShowInput() replaces the two functions
 * ShowInput1() and ShowInput2()).
 *
 * Revision 0.95  1994/10/09  02:32:49  drapeau
 * Several changes, having to do with output:
 *
 * * Reorganized OutputCallback() to work in the style of Input1 and
 *   Input2 callbacks; that is, to call a XtWorkProc to do the actual
 *   work.  This is because the output callback now deletes and creates
 *   new windows, and this does not happen immediately in an Xt callback.
 *
 * * In the Output callback work proc, call OutputSelect with the special
 *   channel "PLX_OUTPUT_JPEG_OFF" for the imageQuality widget even
 *   though output is not applied to it.  This hack for the HP explictly
 *   removes XvPortID-to-Window mappings at the XPlx library layer, so
 *   that when the widgets are later deleted and re-created (via
 *   CreateJPEGWidget()), the widgets can allocate Xv Ports for
 *   themselves from scratch and no Ports will be left in the XPlx
 *   layer's internal list of Port-to-Window mappings.
 *
 * Revision 0.94  1994/10/06  02:20:55  drapeau
 * Removed extraneous code from Input1WorkProc() that reset the record
 * speed to the full frame rate for the video standard in effect (30 fps
 * for NTSC, 25 for PAL or SECAM).
 *
 * Revision 0.93  1994/10/03  17:25:50  drapeau
 * Two minor changes:
 *
 * 1) Don't allow output to be used until a JPEG movie is ready to be
 *    played (either one has just been recorded or one has just been
 *    opened for playback).
 *
 * 2) In CreateJPEGWidget(), managed the child of the imageQualityFrame
 *    widget after the new one has been created.
 *
 * Revision 0.92  1994/09/30  17:01:31  drapeau
 * Added support for output.  This version of the module is pretty badly
 * hacked; the implementation is not at all clean.  It would be better if
 * the JPEG widgets did not have to be destroyed and re-created in order
 * to get output working correctly.  Currently they have to be destroyed
 * because it is the only way to detach Xv Ports from their drawables.
 * It would be better if Xv provided a way to let you manually detach a
 * Port from a drawable.
 *
 * Revision 0.91  1994/09/13  08:09:58  drapeau
 * Removed extraneous variables.
 *
 * Revision 0.90  1994/08/27  05:49:02  drapeau
 * First working version.
 * */
static char hpSpecificRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/hpSpecific.c,v 0.114 1996/06/12 22:34:46 lovvik Exp $";

