/* $Header: /plx/source/Xclients/generic/RCS/callbacks.c,v 1.89 1996/10/30 02:39:19 lovvik Exp $ */
/* $Log: callbacks.c,v $
 * Revision 1.89  1996/10/30  02:39:19  lovvik
 *   Improved multi-board support.  Specifically, I fixed some minor
 *   problems in Input1Callback().
 *
 * Revision 1.88  1996/10/03  17:36:10  lovvik
 *   Added a call to XCloseDisplay() in QuitWorkProc() so I could
 *   check for library level memory leaks.
 *
 * Revision 1.87  1996/09/04  00:25:45  lovvik
 *   Modified ZoomWorkProc().  When the zoom factor was changed and the output widget
 *   was showing one of the input windows, the output widget would display an
 *   error message.
 *
 * Revision 1.86  1996/08/30  23:07:42  lovvik
 *   Output changes:  Selecting Any|4:3 aspect ratio would reset
 *   the origin of the output.  This has now been fixed.
 *
 * Revision 1.85  1996/08/26  22:37:14  lovvik
 *   Modified Input1Callback.  If output is on and tracking the
 *   Input 1 window, output will turn off when the user turns off
 *   Input 1.
 *
 * Revision 1.84  1996/08/07  16:47:45  lovvik
 *   Removed call to RestoreInput2IfNeeded().  This was a hack meant
 *   to overcome a problem in Parallax libraries.  The library has
 *   now been fixed, so this is not necessary.
 *
 * Revision 1.83  1996/08/02  00:39:02  lovvik
 *   Fixed a bug in ZoomWorkProc:  If the second video widget
 *   was selected, the first video widget would not resize on
 *   a zoom change.
 *
 * Revision 1.82  1996/06/26  16:19:49  lovvik
 *   Changed all references of zoomFactor.  zoomFactor is now
 *   an array of zoom factor data.  Also modified ZoomWorkProc()
 *   so it works with multiple boards.  (Before if multiple boards
 *   were used and the zoom factor changed, all window geometries
 *   changed.  Now, only the windows associated with the particular
 *   board are modified.)
 *
 * Revision 1.81  1996/06/12  21:48:22  lovvik
 *   Added code to ShowOutputRectangle() that disables window
 *   tracking.  Before, the window would continue to be tracked
 *   by the output widget even after the user selected a new
 *   output rectangle.
 *
 * Revision 1.80  1996/05/14  21:31:46  lovvik
 *   Fixed a problem in SaveImageFile().  The call to
 *   SaveImageFileInFormat() did not correctly deal with
 *   the current zoom factor.
 *
 * Revision 1.79  1996/05/07  16:55:33  lovvik
 *   Added code to OpenImageFile() to detect if the hardware
 *   supports JPEG decompression before attempting to decompress
 *   a JFIF or JPEG image.
 *
 * Revision 1.78  1996/05/01  21:01:48  lovvik
 *   Fixed a problem in SaveImageFile().  In zoom mode, some
 *   images were saving too much data.
 *
 * Revision 1.77  1996/04/21  23:41:11  lovvik
 *   Added multiple board support.
 *
 * Revision 1.76  1996/03/08  18:06:44  lovvik
 *   Modified ShowInputSettings to support up to 8 inputs.
 *
 * Revision 1.75  1996/01/25  04:21:25  drapeau
 * Fixed minor inconsistencies in interface management: the state changes
 * for the two "Live/Capture..." buttons should be affected by both
 * clicking on them and by shift-clicking in their respective video
 * windows.  The shift-click was not affecting the state changes for the
 * buttons.
 *
 * Revision 1.74  1996/01/22  20:44:59  drapeau
 * Enhanced code to manage the "Capture..." buttons more robustly.
 *
 * Revision 1.73  1996/01/22  07:57:17  drapeau
 * Made support for the "stillDelay" resource more robust.
 *
 * Revision 1.72  1996/01/22  07:31:41  drapeau
 * Added support for a new XVideoIn widget resource, "stillDelay".  The
 * new resource allows the programmer to set the delay time between the
 * time the widget freezes the digitizer and the time the widget grabs a
 * still image from its window.
 *
 * Revision 1.71  1996/01/18  19:20:30  drapeau
 * Modified SaveStill() to remove the file selection box when a video
 * window goes from a still back to a live state.
 *
 * Revision 1.70  1996/01/08  21:48:51  drapeau
 * Added code to check if the 2nd video input widget has been created
 * before trying to operate on it.
 *
 * Revision 1.69  1996/01/05  21:02:29  drapeau
 * Two changes:
 *
 * 1) Added a function SaveStill() to allow for quicker and easier saving
 * of still images.  The new functionality allows you to click on either
 * of the Video Input windows and instantly grab a still image.
 *
 * 2) Fixed reporting in the ShowInputSettings() function: there had been
 * no case to handle YUV inputs; now there is.
 *
 * Revision 1.68  1995/12/20  02:26:49  drapeau
 * Enhanced the autoSearch method, especially in the SetSignalStandard()
 * function.  The new autoSearch method now allows the application
 * programmer to specify a specific digitizer on which to do autoSearch.
 * The previous version of the autoSearch feature in the XVideoIn widget
 * only allowed autoSearch to be performed on all digitizers.
 *
 * Revision 1.67  1995/12/14  23:41:10  lovvik
 * Made 2 changes:
 *   1.  Changed SaveImageFile() so it verifies that
 *       the specified file can be written to before
 *       trying to save an image.
 *   2.  Changed OpenImageFile() to print an error
 *       message if the specified file is not a JPEG
 *       file or a JFIF file.
 *
 * Revision 1.66  1995/12/14  03:01:34  drapeau
 * Fixed the SaveImageFile() function to check for NULL JPEG images after
 * a compression request is issued.
 *
 * Revision 1.65  1995/10/25  01:08:32  drapeau
 * Fixed SetAnyAspectRatio() so that 4-to-3 aspect ratio suggestions to
 * the window manager are undone.  Any aspect ratio resizing now works
 * correctly again.
 *
 * Revision 1.64  1995/10/11  02:28:33  drapeau
 * (From Paul Lovvik) Fixed an error in the way video widgets resize
 * under the OPEN LOOK Window Manager.  The code assures that a 4-to-3
 * aspect ratio is maintained, which has the side effect of assuring that
 * live video windows always remain the full height they should be, as
 * opposed to part of the full height because the window manager is not
 * honoring resize requests.
 *
 * Revision 1.63  1995/10/05  04:33:44  drapeau
 * Removed obsolete code now replaced by the OutputCascade functions for
 * selecting output signal types.
 *
 * Also added functionality to allow selecting a specific window for
 * output as opposed to allowing only a region of the display as output.
 *
 * Revision 1.62  1995/08/16  05:02:22  drapeau
 * Modified functions that open a JPEG image of some sort to take into
 * account differences in the behavior of the Motif Window Manager on AIX
 * and HP platforms.  Additional size hints are necessary for the AIX
 * implementation of MWM (see notes for version 1.28 of the "fileUtils.c"
 * module of the MovieTool application for a more complete explanation).
 *
 * Revision 1.61  1995/06/23  22:41:15  drapeau
 * Modified OpenImageFile() so that the JPEG image shell conforms exactly
 * to the size of the JPEG image it is displaying.  The shell is not
 * allowed to resize.
 *
 * Revision 1.60  1995/06/23  19:21:57  drapeau
 * Fixed ZoomWorkProc() so that it correctly saves and restores all
 * relevant video widget values when destroying and creating the new
 * video widget for the new zoom level.
 *
 * Revision 1.59  1995/06/15  23:08:51  drapeau
 * Fixed an error in the ZoomCallback() function: when two live video
 * windows are showing and Zoom is called for, the 1st video window began
 * showing the 2nd video window's input signal.  This is because the
 * ZoomCallback() function destroyed the input widgets and created new
 * ones, but the design of the Xt toolkit prevents widgets from being
 * deleted immediately when in a callback.  The result was that the Xv
 * Ports attached to the video windows were not released reliably, so a
 * race condition occured when creating the new widgets.
 *
 * The fix was to make the ZoomCallback() put most of the actual work in
 * a WorkProc, which does destroy widgets immediately in the function.
 *
 * Revision 1.58  1995/06/01  22:50:19  drapeau
 * Minor modification to ShowInputSettings(): changed the reporting of
 * the instance number so that it matches the numbering scheme of the
 * cascading input menu (which starts at "1" instead of "0").
 *
 * Revision 1.57  1995/05/31  18:56:52  drapeau
 * Two minor changes:
 *
 * 1) Modified Input1Callback() so that it calls ShowInputSettings()
 *    after a new channel is selected.  This is to insure that the
 *    interface always shows the most current information.
 *
 * 2) Modified ShowInputSettings() so that it only updates the input
 *    cascade menu if the current channel is not PLX_IO_OFF, which would
 *    cause the cascade menu to attempt to access a widget that is not
 *    created.
 *
 * Revision 1.56  1995/05/25  18:08:02  drapeau
 * Added code to ShowInputSettings() to display the current signal type
 * in use for each video window.
 *
 * Revision 1.55  1995/05/19  01:55:51  drapeau
 * Added interface code to indicate the current zoom ratio.
 *
 * Revision 1.54  1995/05/15  16:54:47  drapeau
 * Added support for new modules that automatically create cascading
 * menus to reflect the input choices available on the Parallax hardware
 * being used.
 *
 * Also added the ZoomCallback() function for better support of hardware
 * zoom.
 *
 * Revision 1.53  1995/05/02  20:34:51  drapeau
 * Added support for saving and displaying zoomed JPEG images.
 *
 * Revision 1.52  1995/04/18  19:01:51  drapeau
 * Added support for second YC input for HP.  This change should also
 * apply to the IBM product.
 *
 * Revision 1.51  1995/04/01  01:27:13  drapeau
 * Modified WMDeleteWindowCallback() to handle the case when the
 * application is quit from the window manager's title bar menu as
 * opposed to the "Quit" menu item in the application's File menu.
 *
 * Revision 1.50  1995/03/22  00:22:56  drapeau
 * Fixed QuitWorkProc() so that it destroys only non-NULL widgets.
 *
 * Revision 1.49  1994/10/14  21:13:02  drapeau
 * Added simple interface features to gray-out input types not valid for
 * the input being configured.
 *
 * Revision 1.48  1994/10/14  03:07:40  drapeau
 * Two changes:
 *
 * 1) Fixed an error when saving new JFIF images: the Image Shell window
 *    now correctly resizes to reflect the size of the newly-saved image.
 *    Previously, the user had to explicitly open the new file from the
 *    File menu to get the Image Shell to resize.
 *
 * 2) Added code to allow the user either to select an input source
 *    explicitly (disabling auto-search) or to select the "first
 *    available" input source (enabling auto-search).
 *
 * Revision 1.47  1994/10/06  21:28:06  drapeau
 * Added a call to a new HP hack function, RestoreInput2IfNeeded(), that
 * restores the 2nd video input to a working state if it has been stilled
 * and a compressed image was taken from it.
 *
 * Revision 1.46  1994/10/06  02:09:49  drapeau
 * Added logic to the code to force the Input Settings panel to show the
 * name of the currently-selected video widget, overriding the Motif
 * option menu's choice of label whenever possible.
 *
 * Revision 1.45  94/09/30  22:08:37  22:08:37  drapeau (George Drapeau)
 * Fixed program logic errors that prevented S-VHS signals from being
 * selected in some cases.
 * 
 * Revision 1.44  1994/09/30  16:53:51  drapeau
 * Added two parameters to SaveImageFileInFormat(), to accommodate saving
 * images in other than full-size (width & height).
 *
 * Revision 1.43  1994/09/25  07:30:22  drapeau
 * Enhanced code for indicating the currently selected video widget and
 * which input widget's settings to show in the Input Settings panel.
 *
 * Revision 1.42  1994/09/23  06:31:57  drapeau
 * Putting in working versions of a new interface that allows users to
 * select explicitly any signal type from the Input Settings panel.
 *
 * Also new code to detect when inputs are enabled and disabled.  Some
 * functions (such as still image saving) are not allowed when inputs are
 * disabled.
 *
 * Revision 1.41  1994/09/14  05:58:24  drapeau
 * Significant changes to the implementation of the rubberband code.  It
 * works cleanly now.
 *
 * Revision 1.40  1994/09/14  03:20:19  drapeau
 * Added functionality to ShowInputSettings() to properly display the
 * current input signal format and standard.  Moved this functionality
 * from other functions where it does not really belong.
 *
 * Revision 1.39  1994/09/13  08:02:47  drapeau
 * Several changes:
 *
 * * Fixed the Quit() function so that it correctly destroys video
 *   widgets.
 *
 * * Added function "ResizeVideoWidgetIfNecessary()" to resize input
 *   widgets if the input signal characteristics have changed in such a
 *   way as to necessitate resizing of the widget (e.g., if signal std
 *   was changes from NTSC to PAL).
 *
 * * Implemented callbacks to handle output aspect ratio changes.
 *
 * Revision 1.38  1994/09/04  07:15:17  drapeau
 * Added WMDeleteWindowCallback(), a function that is called when the
 * window manager tries to close one of the video input shells.  The
 * function is responsible for updating the rest of the interface
 * correctly (namely, the video input toggles on the main panel).
 *
 * Revision 1.37  1994/09/03  06:01:55  drapeau
 * Modified code to better reflect the current state of video input
 * channels.  Now when trying to select a new input source, the
 * application now queries the video input widget to determine whether
 * the new input selection actually took place.
 *
 * Revision 1.36  1994/08/29  22:56:37  drapeau
 * Removed problem code in setting 4:3 and any aspect ratios: the code
 * used to set the widthInc and heightInc resources for the parent shell,
 * but this caused problems with the VUE window manager (it might've with
 * MWM, too; didn't check) and solved nothing.  In short, the code was
 * extraneous and troublesome, and so was removed.
 *
 * Revision 1.35  1994/08/27  05:19:33  drapeau
 * Enhanced Input1Callback() so that it explicitly turns on video when
 * set.  This is to handle the case when there was previously no sync
 * signal; now, the user only need toggle Input1 off then on again to try
 * to look again for sync on that channel.
 *
 * Also, added support for displaying error messages in a dialog box
 * instead of to stdout.
 *
 * Revision 1.34  1994/08/23  05:39:57  drapeau
 * Removed extraneous printf's.
 *
 * Revision 1.33  1994/08/10  03:10:47  drapeau
 * Moved Input2Callback() and OutputCallback() to the os-specific files,
 * since these two functions sensitive to platform-specific rules.
 *
 * Revision 1.32  1994/08/01  18:34:17  drapeau
 * Added functionality to better facilitate input selection, to conform
 * with the new model that accounts for multiple instances of the same
 * type of video signal.
 *
 * Revision 1.31  1994/06/23  20:59:11  drapeau
 * Added two functions to cleanly handle shell popdowns when a video input
 * shell is popped down from the window manager menu instead of from the main
 * panel.
 *
 * Revision 1.3  1994/06/14  00:39:30  drapeau
 * Fixed an error in the way JFIF files are saved.  When saving a
 * PAL-sized image, the source compression rectangle was previously not
 * correctly adjusted from NTSC size.
 *
 * Revision 1.2  1994/03/22  04:53:07  drapeau
 * Cosmetic changes to clean up the code, reflecting a restructuring of the
 * application with the addition of a "VideoTool.h" module for all major
 * includes and data structure definitions.  The "globals.h" module is now
 * obsolete.
 *
 * Revision 1.1  1993/12/01  03:03:18  drapeau
 * Initial revision
 * */
static char callbacksRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/callbacks.c,v 1.89 1996/10/30 02:39:19 lovvik Exp $";
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
#include "VideoTool.h"

extern		XPlxHardwareInfo hardwareInfo;

XPlxCImage*	ReadJPEGFile(char*, XPlxCImage*);
static void	DisplayVideoEffectsOptionSetting(Widget);
void		OutputShellEventHandler(Widget, XtPointer, XConfigureEvent*);
void 		ChangeStillButtonLabel(Widget w, Boolean live);



void XDmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtManageChild ( *(Widget *)client_data );
}

void XDunmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtUnmanageChild ( *(Widget *)client_data );
}

void XDpopup_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtPopup ( *(Widget *)client_data, XtGrabNone );
}

void XDpopdown_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtPopdown ( *(Widget *)client_data );
}

void XDmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtMapWidget (*(Widget *)client_data);
}

void XDunmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtUnmapWidget (*(Widget *)client_data);
}

void XDenable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtSetSensitive (*(Widget *)client_data, TRUE);
}

void XDdisable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtSetSensitive (*(Widget *)client_data, FALSE);
}

void ProcessAllXtRequests(void)
{
  XEvent event;  
  while(XtAppPending(app_context))
  {
    XtAppNextEvent(app_context, &event);
    XtDispatchEvent(&event);
  }
}

void 
  SetStillVideo (Widget		w,
		 XtPointer	client_data,
		 XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;

  XtVaSetValues(selectedVideoWidget, XtNlive, False, NULL);
  return;
}								    /* end function SetStillVideo */


void
  LiveStillVideoChanged(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct* callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  Widget whichLabelWidget;

  if(callData->set == False)
  {
    /*
     * Ignore this - the button is being turned off.
     */
    return;
  }
  if (selectedVideoWidget == input1Widget)			    /* Determine which "Capture.../Live" widget to update */
    whichLabelWidget = stillInput1Button;
  else
    whichLabelWidget = stillInput2Button;

  if(w == liveVideoToggle)
  {
    XtVaSetValues(selectedVideoWidget, XtNlive, True, NULL);
    ChangeStillButtonLabel(whichLabelWidget, True);
    return;
  }
  if(w == stillVideoToggle)
  {
    XtVaSetValues(selectedVideoWidget, XtNlive, False, NULL);
    ChangeStillButtonLabel(whichLabelWidget, False);
    return;
  }
}

void
  SqueezeCropVideoChanged(Widget		w,
			  XtPointer		client_data,
			  XtPointer		xt_call_data)
{
  XmToggleButtonCallbackStruct* callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;

  if(callData->set == False)
  {
    /*
     * Ignore this - the button is being turned off.
     */
    return;
  }
  if(w == squeezeVideoToggle)
  {
    XtVaSetValues(selectedVideoWidget, XtNcropped, False, NULL);
    return;
  }
  if(w == cropVideoToggle)
  {
    XtVaSetValues(selectedVideoWidget, XtNcropped, True, NULL);
    return;
  }
}


void
  AspectRatioChanged(Widget		w,
		     XtPointer		client_data,
		     XtPointer		xt_call_data)
{
  XmToggleButtonCallbackStruct* callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;

  if(callData->set == False)
  {
    /*
     * Ignore this - the button is being turned off.
     */
    return;
  }
  if(w == anyAspectRatioToggle)
  {
    XtVaSetValues(selectedVideoWidget, XtNnormalAspectRatio, False, NULL);
    if (selectedVideoWidget == input1Widget)
    {
      XtVaSetValues(input1Shell,					    /* Allow the shell to be resized to any aspect ratio again */ 
		    XtNminAspectX, XtUnspecifiedShellInt,
		    XtNminAspectY, XtUnspecifiedShellInt,
		    XtNmaxAspectX, XtUnspecifiedShellInt,
		    XtNmaxAspectY, XtUnspecifiedShellInt,
		    NULL);
    }
    else
    {
      XtVaSetValues(input2Shell,
		    XtNminAspectX, XtUnspecifiedShellInt,
		    XtNminAspectY, XtUnspecifiedShellInt,
		    XtNmaxAspectX, XtUnspecifiedShellInt,
		    XtNmaxAspectY, XtUnspecifiedShellInt,
		    NULL);
    }
    return;
  }
  if(w == fourToThreeAspectRatioToggle)
  {
    XtVaSetValues(selectedVideoWidget, XtNnormalAspectRatio, True, NULL);
    if (selectedVideoWidget == input1Widget)
    {
      XtVaSetValues(input1Shell,					    /* Make sure that the input shell conforms to a 4-to-3... */
		    XtNminAspectX, 4,				    /* ...aspect ratio.  This is necessary for dealing with... */
		    XtNminAspectY, 3,				    /* ...the OPEN LOOK window manager (the Motif window... */
		    XtNmaxAspectX, 4,				    /* ...manager doesn't require this restriction since it... */
		    XtNmaxAspectY, 3,				    /* ...listens to the XVideoIn widget's resize requests). */
		    NULL);
    }
    else
    {
      XtVaSetValues(input2Shell,
		    XtNminAspectX, 4,
		    XtNminAspectY, 3,
		    XtNmaxAspectX, 4,
		    XtNmaxAspectY, 3,
		    NULL);
    }
    return;
  }
}


void 
  SetBrightness(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNbrightness, callData->value, NULL);
  return;
}								    /* end function SetBrightness */



void 
  SetContrast(Widget	w,
	      XtPointer client_data,
	      XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNcontrast, callData->value, NULL);
  return;
}								    /* end function SetContrast */


void 
  SetHue(Widget		w,
	 XtPointer	client_data,
	 XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNhue, callData->value, NULL);
  return;
}								    /* end function SetHue */



void 
  SetSaturation(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNsaturation, callData->value, NULL);
  return;
}								    /* end function SetSaturation */



Boolean QuitWorkProc(XtPointer unusedClientData)
{
  Display* display;

  display = XtDisplay(mainWindow);
  XtDestroyWidget(input1Widget);
  if ((Widget)NULL != input2Widget)				    /* Was a 2nd input widget created (i.e., VIO board used)? */
    XtDestroyWidget(input2Widget);				    /* Yes, destroy it */
  if ((Widget)NULL != outputWidget)				    /* Was an output widget created (i.e., VIO board used)? */
    XtDestroyWidget(outputWidget);				    /* Yes, destroy it */
  if ((Widget)NULL != jpegWidget)				    /* Was a JPEG widget created? */
    XtDestroyWidget(jpegWidget);				    /* Yes, destroy it */
  XtDestroyWidget(mainWindow);					    /* Free resources taken by all widgets in this application */
  XCloseDisplay(display);
  exit(0);
} /* end function QuitWorkProc */



void Quit(Widget	w,
	  XtPointer	client_data,
	  XtPointer	xt_call_data)
{
  XtWorkProcId	quitProcID;
  
  quitProcID = XtAppAddWorkProc(app_context,			    /* Start a work-proc to quit the application; Xt is... */
				QuitWorkProc,			    /* ...poorly designed and this is the only way to assure... */
				(XtPointer)NULL);		    /* ...that shared memory will be freed from the JPEG... */
  return;							    /* ...widgets. */
} /* end function Quit */



void Input1Callback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  static int			lastStandard = PLX_NTSC;
  int				signalStandard;
  Boolean			outputOn;
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)
  {
    if(((currentInput1Channel & 0x0000ffff) == PLX_INPUT_1) &&
       ((currentInput1Channel >> 16) == (PlxOutputToChannel[defaultOutputChannel]>>16)))
    {
      /*
       * Input 1 and output will conflict.
       */
      XtVaGetValues(videoOutToggle,
		    XmNset, &outputOn,
		    NULL);

      if(outputWidget != (Widget)NULL && outputOn)
      {
	/*
	 * Output is on, so we must turn it off.
	 */
	XmToggleButtonSetState(videoOutToggle, False, True);
	ProcessAllXtRequests();
      }
    }

    XtPopup(input1Shell, XtGrabNone);
    XtVaSetValues(videoIn1Toggle, XmNlabelString, onString, NULL);
    XtVaSetValues(input1Widget,
		  XtNchannel, currentInput1Channel,
		  NULL);	    /* Restart video input if possible */
    XtVaGetValues(input1Widget,					    /* Determine if an active signal is on input 1 and... */
		  XtNsync, &input1Sync,				    /* ...the new signal standard for the widget, if any. */
		  XtNsignalStandard, &signalStandard,
		  NULL);
    ShowInputSettings(input1ChannelButton, (XtPointer)NULL,	    /* Update the current input settings now that a new... */
		      (XtPointer)NULL);				    /* ...input signal has been selected. */
    if (input1Sync == False)					    /* Sync wasn't found, don't bother popping up the shell */
    {
      XmStringFree(onString);
      XmStringFree(offString);
      return;
    }
    XtVaSetValues(stillInput1Button, XmNsensitive, True, NULL);	    /* Allow capture on the 1st video input window */
    if (lastStandard != signalStandard)				    /* Did the signal standard for this widget change? */
      ResizeVideoWidgetIfNecessary(input1Widget);		    /* Yes, widget might need resizing */
    lastStandard = signalStandard;
  }
  else
  {
    if(OutputAttachedToInputWidget(input1Widget))
    {
      /*
       * Need to turn off output.
       */
      XmToggleButtonSetState(videoOutToggle, False, True);
    }
    XtPopdown(input1Shell);
    XtVaSetValues(videoIn1Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(input1Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off video input 1 */
    XtVaSetValues(stillInput1Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 1st video input window */
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function Input1Callback */



void 
  PopupConfigPanel(Widget		w,
		   XtPointer	client_data,
		   XtPointer	xt_call_data)
{
  static Boolean		widgetIsManaged = False;
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if (widgetIsManaged == False)					    /* Check if the shell has been managed yet; if not, do so... */
  {								    /* ...so that it will properly display. */
    XtManageChild(configShell);
    widgetIsManaged = True;
  }
  if (XtIsRealized(configShell) == False)
    XtRealizeWidget(configShell);
  XtPopup(configShell, XtGrabNone);
  return;
}								    /* end function PopupConfigPanel */



void 
  ShowInputSettings(Widget	w,
		    XtPointer	clientData,
		    XtPointer	callData)
{
  static GC	hardwareGC = NULL;
  int		brightness, saturation, contrast, hue;
  int		channel;
  int		instance;
  XmString	label;
  int		signalFormat;
  int		signalStandard;
  Boolean	live;
  Boolean	cropped;
  Boolean	normalAspectRatio;
  Boolean	sync;
  signalSpec	theSignal;
  char		standardString[8];
  char		formatString[16];
  char		instanceString[3];
  char		channelString[16];
  char		syncString[16];
  char		inputStatusString[64];
  char		titleBarString[64];
  XPlxHardwareInfo	hwInfo;

  if(hardwareGC == NULL)
  {
    hardwareGC = XCreateGC(XtDisplay(selectedVideoWidget),
			   XtWindow(selectedVideoWidget),
			   0,
			   NULL);
  }
  if (w == input1ChannelButton)					    /* Determine which video widget's settings are to be shown */
  {
    SelectVideoWidget(input1Widget);
    selectedSettingsWidget = input1ChannelButton;
    label = XmStringCreateLocalized("Window 1");
    XtVaSetValues(channelMenu, XmNlabelString, label, NULL);
    XtVaSetValues(channelOptionButton, XmNlabelString, label, NULL);
    XmStringFree(label);
  }
  else
  {
    SelectVideoWidget(input2Widget);
    selectedSettingsWidget = input2ChannelButton;
    label = XmStringCreateLocalized("Window 2");
    XtVaSetValues(channelMenu, XmNlabelString, label, NULL);
    XtVaSetValues(channelOptionButton, XmNlabelString, label, NULL);
    XmStringFree(label);
  }
  XtVaGetValues(selectedVideoWidget,				    /* Retrieve current state of the appropriate video widget */
		XtNbrightness, &brightness,
		XtNsaturation, &saturation,
		XtNcontrast, &contrast,
		XtNhue, &hue,
		XtNlive, &live,
		XtNcropped, &cropped,
		XtNnormalAspectRatio, &normalAspectRatio,
		XtNinstance, &instance,
		XtNsignalFormat, &signalFormat,
		XtNsignalStandard, &signalStandard,
		XtNchannel, &channel,
		XtNsync, &sync,
		NULL);
  theSignal.s.format = signalFormat;
  theSignal.s.instance = instance;
  theSignal.s.standard = signalStandard;
  theSignal.s.channel = channel;
  DisplayVideoEffectsOptionSetting(selectedVideoWidget);	    /* Make the OptionMenu display the correct video effect */
  XmScaleSetValue(brightnessSlider, brightness);		    /* Set interface elements to reflect the state of the widget */
  XmScaleSetValue(contrastSlider, contrast);
  XmScaleSetValue(hueSlider, hue);
  XmScaleSetValue(saturationSlider, saturation);
  zoomFactor[(channel>>16)] = XPlxGetZoomFactor(XtDisplay(selectedVideoWidget),
						XtWindow(selectedVideoWidget));
  hwInfo = XPlxQueryHardware(XtDisplay(selectedVideoWidget),
			     XtWindow(selectedVideoWidget),
			     hardwareGC);
  if (live == True)						    /* Is the widget set to display live video? */
  {								    /* Yes, update interface components appropriately */
    XmToggleButtonSetState(liveVideoToggle, True, False);
    XmToggleButtonSetState(stillVideoToggle, False, False);
  }
  else
  {
    XmToggleButtonSetState(liveVideoToggle, False, False);
    XmToggleButtonSetState(stillVideoToggle, True, False);
  }
  if (cropped == True)						    /* Is the widget set to crop video? */
  {								    /* Yes, update interface components appropriately */
    XmToggleButtonSetState(cropVideoToggle, True, False);
    XmToggleButtonSetState(squeezeVideoToggle, False, False);
  }
  else
  {
    XmToggleButtonSetState(cropVideoToggle, False, False);
    XmToggleButtonSetState(squeezeVideoToggle, True, False);
  }
  if (normalAspectRatio == True)				    /* Is the widget set to display video at a 4:3 aspect ratio? */
  {								    /* Yes, update interface components appropriately */
    XmToggleButtonSetState(fourToThreeAspectRatioToggle, True, False);
    XmToggleButtonSetState(anyAspectRatioToggle, False, False);
  }
  else
  {
    XmToggleButtonSetState(fourToThreeAspectRatioToggle, False, False);
    XmToggleButtonSetState(anyAspectRatioToggle, True, False);
  }
  if(zoomFactor[(channel>>16)] == 1)
  {
    XmToggleButtonSetState(zoom1XToggle, True, False);
    XmToggleButtonSetState(zoom2XToggle, False, False);
  }
  else
  {
    XmToggleButtonSetState(zoom1XToggle, False, False);
    XmToggleButtonSetState(zoom2XToggle, True, False);
  }
  if(hwInfo.info.zoom == True)
  {
    XtSetSensitive(zoomCascade, True);
  }
  else
  {
    XtSetSensitive(zoomCascade, False);
  }
  if(hwInfo.info.overlay == True)
  {
    /*
     * Special effects are not available.
     */
    if(effectsOptionLabel != (Widget)NULL)
    {
      XtSetSensitive(effectsOptionLabel, False);
      XtSetSensitive(effectsOptionButton, False);
    }
  }
  else
  {
    if(effectsOptionLabel != (Widget)NULL)
    {
      XtSetSensitive(effectsOptionLabel, True);
      XtSetSensitive(effectsOptionButton, True);
    }
  }
  if(hwInfo.info.numJPEGCompressors > 0)
  {
    /*
     * JPEG compression is available.
     */
    XtSetSensitive(saveAsJFIFButton, True);
    XtSetSensitive(saveAsJPEGButton, True);
  }
  else
  {
    XtSetSensitive(saveAsJFIFButton, False);
    XtSetSensitive(saveAsJPEGButton, False);
  }
  sprintf(instanceString, "%d", instance + 1);			    /* Print the instance number */
  switch (PlxChannelToInput(channel))
  {
   case PLX_INPUT_0:
    sprintf(channelString, "Input 1");
    break;
   case PLX_INPUT_1:
    sprintf(channelString, "Input 2");
    break;
   case PLX_INPUT_2:
    sprintf(channelString, "Input 3");
    break;
   case PLX_INPUT_3:
    sprintf(channelString, "Input 4");
    break;
   case PLX_INPUT_4:
    sprintf(channelString, "Input 5");
    break;
   case PLX_INPUT_5:
    sprintf(channelString, "Input 6");
    break;
   case PLX_INPUT_6:
    sprintf(channelString, "Input 7");
    break;
   case PLX_INPUT_7:
    sprintf(channelString, "Input 8");
    break;
   case PLX_IO_OFF:
    sprintf(channelString, "(Off)");
    break;
  }
  switch (signalFormat)						    /* Print the appropriate label on the popup menu cascade... */
  {								    /* ...button, according to signal format in effect. */
   case PLX_COMP:						    /* One wire: composite video */
    sprintf(formatString, "Composite");
    break;
   case PLX_YC:							    /* Two wires: S-Video */
    sprintf(formatString, "S-VHS (YC)");
    break;
   case PLX_RGB:						    /* Three wires: Red-Green-Blue video */
    sprintf(formatString, "RGB");
    break;
   case PLX_YUV:						    /* Three wires: YUV video */
    sprintf(formatString, "YUV");
    break;
  }
  switch (signalStandard)					    /* Print the appropriate label on the popup menu cascade... */
  {								    /* ...button, according to the actual signal standard... */
   case PLX_NTSC:						    /* ...in effect. */
    sprintf(standardString, "NTSC");
    break;
   case PLX_PAL:
    sprintf(standardString, "PAL");
    break;
   case PLX_SECAM:
    sprintf(standardString, "SECAM");
    break;
  }
  if (True == sync)
    sprintf(syncString, "");
  else
    sprintf(syncString, " (No Sync)");
  sprintf(inputStatusString, "%s : %s : %s : %s%s",
	  channelString,
	  formatString,
	  instanceString,
	  standardString,
	  syncString);
  label = XmStringCreateLocalized(inputStatusString);		    /* Create the string that will be used as a label for the... */
  XtVaSetValues(inputTypeInfoLabel, XmNlabelString, label, NULL);   /* ...Label widget showing the new input information. */
  if (w == input1ChannelButton)
  {
    sprintf(titleBarString, "Video Window 1: %s", inputStatusString);
    XtVaSetValues(input1Shell, XmNtitle, titleBarString, NULL);
    XtVaSetValues(stillInput1Button, XmNsensitive, sync, NULL);	    /* Set capture ability dependent on sync for Input 1 */
  }
  else
  {
    sprintf(titleBarString, "Video Window 2: %s", inputStatusString);
    XtVaSetValues(input2Shell, XmNtitle, titleBarString, NULL);
    XtVaSetValues(stillInput2Button, XmNsensitive, sync, NULL);	    /* Set capture ability dependent on sync for Input 2 */
  }
  XmStringFree(label);
  if (channel != PLX_IO_OFF)					    /* Is the current input a valid input?  (i.e., not off) */
    PlxShowInputSelections(inputMenuCascade, theSignal);	    /* Yes, update the input settings cascade menu */
  return;
}								    /* end function ShowInputSettings */



void 
  RestoreFromUser1(Widget	w,
		   XtPointer	client_data,
		   XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNrestoreColorSettings, USER_0, NULL);
  ShowInputSettings(selectedSettingsWidget, (XtPointer)NULL, (XtPointer)NULL);
  return;
}								    /* end function RestoreFromUser1 */



void 
  RestoreFromUser2(Widget	w,
		   XtPointer	client_data,
		   XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNrestoreColorSettings, USER_1, NULL);
  ShowInputSettings(selectedSettingsWidget, (XtPointer)NULL, (XtPointer)NULL);
  return;
}								    /* end function RestoreFromUser2 */



void 
  RestoreFromFactoryDefaults(Widget	w,
			     XtPointer	client_data,
			     XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNrestoreColorSettings, FACTORY, NULL);
  ShowInputSettings(selectedSettingsWidget, (XtPointer)NULL, (XtPointer)NULL);
  return;
}								    /* end function RestoreFromFactoryDefaults */



void 
  SaveToUser1(Widget	w,
	      XtPointer	client_data,
	      XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNsaveColorSettings, USER_0, NULL);
  return;
}								    /* end function SaveToUser1 */



void 
  SaveToUser2(Widget	w,
	      XtPointer	client_data,
	      XtPointer xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNsaveColorSettings, USER_1, NULL);
  return;
}								    /* end function SaveToUser2 */



void 
  NormalSize(Widget	w,
	     XtPointer	client_data,
	     XtPointer	xt_call_data)
{
  Dimension			width, height;
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaGetValues(selectedVideoWidget,				    /* Determine the full size of this video signal */
		XtNsignalWidth, &width,
		XtNsignalHeight, &height,
		NULL);
  XtVaSetValues(selectedVideoWidget,				    /* Reset the video widget to the full size of the signal... */
		XtNwidth, width,				    /* ...this is done as a side effect of resizing the video... */
		XtNheight, height,				    /* ...widget's parent shell. */
		NULL);
  return;
}								    /* end function NormalSize */



void 
  QuarterSize (Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  Dimension			width, height;
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaGetValues(selectedVideoWidget,				    /* Get the current size of the video widget */
		XtNsignalWidth, &width,
		XtNsignalHeight, &height,
		NULL);
  width = width / 2;
  height = height /2;
  XtVaSetValues(selectedVideoWidget,
		XmNwidth, width,
		XmNheight, height,
		NULL);
  return;
}								    /* end function QuarterSize */


void SelectVideoWidget(Widget theWidget)
{
  selectedVideoWidget = theWidget;
  if (theWidget == input1Widget)
    selectedSettingsWidget = input1ChannelButton;		    /* Determine which settings to show: correspond with the... */
  if (theWidget == input2Widget)				    /* ...currently-selected widget. */
    selectedSettingsWidget = input2ChannelButton;
  return;
}								    /* end function SelectVideoWidget */



void 
  EffectsCallback(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct *callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget,
		XtNvideoParameterString, client_data,
		NULL);
  return;
}								    /* end function EffectsCallback */



static void DisplayVideoEffectsOptionSetting(Widget videoWidget)
{
  Widget	theWidget = (Widget)NULL;
  String	theString;
  
  XtVaGetValues(videoWidget,					    /* Determine the video effect set for this widget */
		XtNvideoParameterString, &theString,
		NULL);
  if (strcmp(theString, "COLOR") == 0)				    /* Determine which widget corresponds to the effect... */
    theWidget = effects1Button;					    /* ...currently being used by this widget */
  if (strcmp(theString, "BW") == 0)
    theWidget = effects2Button;
  if (strcmp(theString, "INVERTED") == 0)
    theWidget = effects3Button;
  if (strcmp(theString, "NONINVERTED") == 0)
    theWidget = effects4Button;
  if (strcmp(theString, "GENLOCK") == 0)
    theWidget = effects5Button;
  if (strcmp(theString, "FREERUN") == 0)
    theWidget = effects6Button;
  XtVaSetValues(effectsOptionMenu,				    /* Update the OptionMenu to display the correct effect */
		XmNmenuHistory, theWidget,
		NULL);
  return;
}								    /* end function DisplayVideoEffectsOptionSetting */


void 
  OpenImageFile (Widget w,
		 XtPointer client_data,
		 XtPointer xt_call_data )
{
  char*					fileName = NULL;
  XPlxCImage*				jpegData = NULL;
  XPlxCImage*				sharedImageData = NULL;
  FILE*					theFile;
  int					fileType;
  int					zFactor;
  Boolean				result;
  Boolean				success = False;
  Boolean				usingSharedMemory = False;
  static Boolean			widgetIsManaged = False;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
  result = XmStringGetLtoR(callData->value,			    /* Try to determine the filename typed by into the... */
			   XmSTRING_DEFAULT_CHARSET,		    /* ...File Selection Box. */
			   &fileName);
  if (result == False)						    /* Could not determine the filename that was typed */
  {
    XtUnmanageChild(fileDialogSelectionBox);
    PrintErrorMessage("The filename you typed included unprintable characters.  Please try again.");
    return;
  }
  theFile = fopen(fileName, "r");				    /* Attempt to open the requested file */
  if (theFile == (FILE*)NULL)					    /* Did the attempt fail? */
  {
    XtUnmanageChild(fileDialogSelectionBox);
    PrintErrorMessage("Could not open the file you specified.");
    return;
  }
  if(selectedVideoWidget == input1Widget)
  {
    zFactor = zoomFactor[0];
  }
  else
  {
    zFactor = zoomFactor[1];
  }
  
  if(XPlxIsJFIFFile(fileName) == True)				    /* Is this file a JFIF file? */
  {								    
    if(hardwareInfo.info.numJPEGCompressors == 0)		    /* Is hardware decompression available? */
    {								    /*   No. */
      PrintErrorMessage("Cannot display JFIF file -- No hardware decompression.");
      XtUnmanageChild(fileDialogSelectionBox);
      return;
    }
    XtVaSetValues(jpegWidget,					    /* Yes, load it into the JPEG widget */
		  XtNjfifInputFile, fileName,
		  NULL);
    XtVaGetValues(jpegWidget,
		  XtNjpegData, &jpegData,
		  NULL);
    displayDestRect.x = 0;
    displayDestRect.y = 0;
    displayDestRect.width = jpegData->width * zFactor;
    displayDestRect.height = jpegData->height * zFactor;
    XtVaSetValues(jpegWidget,
		  XtNwidth, (Dimension)(jpegData->width * zFactor),
		  XtNheight, (Dimension)(jpegData->height * zFactor),
		  XtNdisplayDestRect, &displayDestRect,
		  NULL);
    XtVaSetValues(imageShell,
		  XtNwidth, (Dimension)(jpegData->width * zFactor),
		  XtNheight, (Dimension)(jpegData->height * zFactor),
		  XtNbaseWidth, (Dimension)(jpegData->width * zFactor),
		  XtNbaseHeight, (Dimension)(jpegData->height * zFactor),
		  XmNmaxWidth, (Dimension)(jpegData->width * zFactor),
		  XmNmaxHeight, (Dimension)(jpegData->height * zFactor),
		  XmNminWidth, (Dimension)(jpegData->width * zFactor),
		  XmNminHeight, (Dimension)(jpegData->height * zFactor),
		  NULL);
    success = True;
  }
  else								    /* No, is this file a JPEG file? */
    if (XPlxIsJPEGFile(fileName))
    {	
      if(hardwareInfo.info.numJPEGCompressors == 0)		    /* Is hardware decompression available? */
      {								    /*   No. */
	PrintErrorMessage("Cannot display JPEG file -- No hardware decompression.");
	XtUnmanageChild(fileDialogSelectionBox);
	return;
      }
      XtVaGetValues(jpegWidget,					    /* Yes, load the JPEG file data into the widget */
		    XtNuseSharedMemory, &usingSharedMemory,
		    NULL);
      if (usingSharedMemory == True)				    /* Is the JPEG widget using shared memory? */
      {
	XtVaGetValues(jpegWidget,				    /* Yes, don't try to allocate new memory for the JPEG... */
		      XtNjpegData, &sharedImageData,		    /* ...image to be read in from disk; instead, use the... */
		      NULL);					    /* ...shared memory segment already allocated. */
      }
      else
      {
	sharedImageData = (XPlxCImage*)NULL;
      }
      jpegData = ReadJPEGFile(fileName, sharedImageData);
      if (jpegData != NULL)					    /* Was the read successful? */
      {								    /* Yes, load the new data into the JPEG widget */
	displayDestRect.x = 0;
	displayDestRect.y = 0;
	displayDestRect.width = jpegData->width * zFactor;
	displayDestRect.height = jpegData->height * zFactor;
	if (usingSharedMemory == True)				    /* Is the JPEG widget using shared memory? */
	  XtVaSetValues(jpegWidget,				    /* Yes, don't try to tell the widget to use a different... */
			XtNputNewImage,	True,			    /* ...memory segment; instead, just inform it that the... */
			NULL);					    /* ...shared memory segment has new data to display. */
	else							    /* No, the JPEG widget isn't using shared memory; give... */
	  XtVaSetValues(jpegWidget,				    /* ...the widget new JPEG data to display. */
			XtNjpegData, jpegData,
			NULL);
	XtVaSetValues(jpegWidget,
		      XtNwidth, (jpegData->width * zFactor),
		      XtNheight, (jpegData->height * zFactor),
		      XtNdisplayDestRect, &displayDestRect,
		      NULL);
	XtVaSetValues(imageShell,
		      XtNwidth, (Dimension)(jpegData->width * zFactor),
		      XtNheight, (Dimension)(jpegData->height * zFactor),
		      XtNbaseWidth, (Dimension)(jpegData->width * zFactor),
		      XtNbaseHeight, (Dimension)(jpegData->height * zFactor),
		      XmNmaxWidth, (Dimension)(jpegData->width * zFactor),
		      XmNmaxHeight, (Dimension)(jpegData->height * zFactor),
		      XmNminWidth, (Dimension)(jpegData->width * zFactor),
		      XmNminHeight, (Dimension)(jpegData->height * zFactor),
		      NULL);
	success = True;
      }
    }
    else
    {
      PrintErrorMessage("Warning:  VideoTool cannot display the specified file.");
    }
  if (success == True)						    /* Is it okay to show the widget? */
  {
    if (widgetIsManaged == False)				    /* Yes, display the widget and its image */
    {								    /* Has the shell been managed already? */
      XtManageChild(imageShell);				    /* No, manage it to show it on-screen for the 1st time */
      widgetIsManaged = True;					    /* Set a variable saying that the shell has been managed */
    }
    else							    /* Yes, the shell has already been managed; simply... */
      XtPopup(imageShell, XtGrabNone);				    /* ...popup the shell. */
  }
  XtUnmanageChild(fileDialogSelectionBox);
  return;
}								    /* end function OpenImageFile */



void SaveImageFile(Widget	w,
		   XtPointer	client_data,
		   XtPointer	xt_call_data)
{
  char*					fileName = (char*)NULL;
  XPlxCImage*				jpegData = NULL;
  FILE*					theFile;
  int					fileType;
  int					zFactor;
  Boolean				result;
  Boolean				success = False;
  Dimension				sourceWindowWidth;
  Dimension				sourceWindowHeight;
  XRectangle*				compressRect;
  XRectangle*				compressDestRect;
  Boolean				outputOn;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
  result = XmStringGetLtoR(callData->value,			    /* Try to determine the filename typed by into the... */
			   XmSTRING_DEFAULT_CHARSET,		    /* ...File Selection Box. */
			   &fileName);
  if (result == False)						    /* Could not determine the filename that was typed */
  {
    XtUnmanageChild(fileDialogSelectionBox);
    PrintErrorMessage("The filename you typed included unprintable characters.  Please try again.");
    return;
  }
  theFile = fopen(fileName, "a+");				    /* Is the path and filename valid? */
  if(theFile == (FILE*)NULL)
  {
    sprintf(errorMessage,					    /* No, print an error message and return. */
	    "VideoTool cannot write to %s.  Check Permissions.",
	    fileName);
    PrintErrorMessage(errorMessage);
    return;
  }
  fclose(theFile);
  
  if(selectedVideoWidget == input1Widget)
  {
    zFactor = zoomFactor[0];
  }
  else
  {
    zFactor = zoomFactor[1];
  }

  XtVaGetValues(videoOutToggle,
		XmNset, &outputOn,
		NULL);

  if(outputOn)
  {
    XmToggleButtonSetState(videoOutToggle, False, True);
  }
  XtVaGetValues(selectedVideoWidget,				    /* Determine the current size of the video window from... */
		XtNwidth, &sourceWindowWidth,			    /* ...which JPEG data will be digitized. */
		XtNheight, &sourceWindowHeight,
		NULL);
  if (strcmp(fileFormat, "JFIF") == 0)				    /* Is this file a JFIF file? */
  {								    /* Yes, load it into the JPEG widget */
    XtVaGetValues(jpegWidget,					    /* Get the current rectangle used to determine how large... */
		  XtNcompressSourceRect, &compressRect,		    /* ...a region to compress. */
		  XtNcompressDestRect, &compressDestRect,
		  NULL);
    compressRect->width = (unsigned short) sourceWindowWidth;	    /* Make sure that only the window is compressed, no more,... */
    compressRect->height = (unsigned short)sourceWindowHeight;	    /* ...no less. */
    compressDestRect->width = (unsigned short)sourceWindowWidth;
    compressDestRect->height = (unsigned short)sourceWindowHeight;
    XtVaSetValues(jpegWidget,					    /* Compress the contents of the video still image window */
		  XtNsourceWin, XtWindow(selectedVideoWidget),
		  XtNgetNewImage, True,
		  NULL);
    XtVaGetValues(jpegWidget,					    /* Point to the newly-compressed image */
		  XtNjpegData, &jpegData,
		  NULL);
    if ((XPlxCImage*)NULL == jpegData)				    /* Did the image compression fail? */
    {								    /* Yes, don't bother doing anything else */
      if(outputOn)
      {
	XmToggleButtonSetState(videoOutToggle, True, True);
      }
      return;
    }
    
    if (zFactor != 1)					    /* Fix the actual size of the JPEG data if the zoom... */
    {								    /* ...factor isn't 1:1. */
      jpegData->width = (jpegData->width / zFactor);
      jpegData->height = (jpegData->height / zFactor);
    }
    XtVaSetValues(jpegWidget,					    /* Save the newly-created JPEG data into a JFIF file */
		  XtNjfifOutputFile, fileName,
		  NULL);
    displayDestRect.x = 0;
    displayDestRect.y = 0;
    displayDestRect.width = jpegData->width * zFactor;
    displayDestRect.height = jpegData->height * zFactor;
    XtVaSetValues(jpegWidget,
		  XtNwidth, (jpegData->width * zFactor),
		  XtNheight, (jpegData->height * zFactor),
		  XtNdisplayDestRect, &displayDestRect,
		  XtNputNewImage, True,
		  NULL);
    XtVaSetValues(imageShell,
		  XtNwidth, (Dimension)(jpegData->width * zFactor),
		  XtNheight, (Dimension)(jpegData->height * zFactor),
		  XtNbaseWidth, (Dimension)(jpegData->width * zFactor),
		  XtNbaseHeight, (Dimension)(jpegData->height * zFactor),
		  XmNmaxWidth, (Dimension)(jpegData->width * zFactor),
		  XmNmaxHeight, (Dimension)(jpegData->height * zFactor),
		  XmNminWidth, (Dimension)(jpegData->width * zFactor),
		  XmNminHeight, (Dimension)(jpegData->height * zFactor),
		  NULL);
    success = True;
  }
  else								    /* No, is this file a JPEG file? */
    if (strcmp(fileFormat, "JPEG") == 0)
    {								    /* Yes, load the JPEG file data into the widget */
      XtVaGetValues(jpegWidget,					    /* Get the current rectangle used to determine how large... */
		    XtNcompressSourceRect, &compressRect,	    /* ...a region to compress. */
		    XtNcompressDestRect, &compressDestRect,
		    NULL);
      compressRect->width = (unsigned short) sourceWindowWidth;	    /* Make sure that only the window is compressed, no more,... */
      compressRect->height = (unsigned short) sourceWindowHeight;   /* ...no less. */
      compressDestRect->width = (unsigned short) sourceWindowWidth;
      compressDestRect->height = (unsigned short) sourceWindowHeight;
      XtVaSetValues(jpegWidget,					    /* Compress the contents of the video still image window */
		    XtNsourceWin, XtWindow(selectedVideoWidget),
		    XtNgetNewImage, True,
		    NULL);
      XtVaGetValues(jpegWidget,					    /* Point to the newly-compressed image */
		    XtNjpegData, &jpegData,
		    NULL);
      if ((XPlxCImage*)NULL == jpegData)			    /* Did the image compression fail? */
      {								    /* Yes, don't bother doing anything else */
	if(outputOn)
	{
	  XmToggleButtonSetState(videoOutToggle, True, True);
	}
	return;
      }

      if (zFactor != 1)						    /* Fix the actual size of the JPEG data if the zoom... */
      {								    /* ...factor isn't 1:1. */
	jpegData->width = (jpegData->width / zFactor);
	jpegData->height = (jpegData->height / zFactor);
      }
      displayDestRect.x = 0;
      displayDestRect.y = 0;
      displayDestRect.width = jpegData->width * zFactor;
      displayDestRect.height = jpegData->height * zFactor;
      XtVaSetValues(jpegWidget,
		    XtNwidth, (jpegData->width * zFactor),
		    XtNheight, (jpegData->height * zFactor),
		    XtNdisplayDestRect, &displayDestRect,
		    XtNputNewImage, True,
		    NULL);
      XtVaSetValues(imageShell,
		    XtNwidth, (Dimension)(jpegData->width * zFactor),
		    XtNheight, (Dimension)(jpegData->height * zFactor),
		    XtNbaseWidth, (Dimension)(jpegData->width * zFactor),
		    XtNbaseHeight, (Dimension)(jpegData->height * zFactor),
		    XmNmaxWidth, (Dimension)(jpegData->width * zFactor),
		    XmNmaxHeight, (Dimension)(jpegData->height * zFactor),
		    XmNminWidth, (Dimension)(jpegData->width * zFactor),
		    XmNminHeight, (Dimension)(jpegData->height * zFactor),
		    NULL);
      if (jpegData != (XPlxCImage*)NULL)			    /* Was the compression step successful? */
      {
	SaveJPEGMovieStill(jpegData, fileName);			    /* Yes, save the JPEG data to a JPEG Movie file */
	success = True;
      }
    }
    else							    /* No, assume the file is a Sun raster file */
    {
      SaveImageFileInFormat(XtDisplay(selectedVideoWidget),
			    XtWindow(selectedVideoWidget),
			    (sourceWindowWidth / zFactor),
			    (sourceWindowHeight / zFactor),
			    fileFormat,
			    fileName);
    }
  XtUnmanageChild(fileDialogSelectionBox);
  if(outputOn)
  {
    XmToggleButtonSetState(videoOutToggle, True, True);
  }
  return;
}								    /* end function SaveImageFile */



void 
  ShowOpenFileDialog (Widget w,
		      XtPointer client_data,
		      XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(fileDialog,
		XmNtitle, "Open",
		NULL);
  XtVaSetValues(saveAsCascadeButton,				    /* This function auto-detects image format; it makes no... */
		XmNsensitive, False, NULL);			    /* ...sense to allow a choice to be made manually. */
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, OpenImageFile,NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtAddCallback(fileDialogSelectionBox, XmNokCallback, OpenImageFile,NULL);
  XtManageChild(fileDialogSelectionBox);
  return;
}								    /* end function ShowOpenFileDialog */



void ShowSaveFileDialog(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  XtVaSetValues(saveAsCascadeButton, XmNsensitive, True, NULL);	    /* Allow manual choice of image format for saving files. */
  XtVaSetValues(fileDialog,
		XmNtitle, "Save",
		NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, OpenImageFile,NULL);
  XtAddCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtManageChild(fileDialogSelectionBox);
  return;
}								    /* end function ShowSaveFileDialog */



void 
  ShowSaveAsFileDialog (Widget w,
			XtPointer client_data,
			XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(saveAsCascadeButton, XmNsensitive, True, NULL);	    /* Allow manual choice of image format for saving files. */
  XtVaSetValues(fileDialog,
		XmNtitle, "Save As",
		NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, OpenImageFile,NULL);
  XtAddCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtManageChild(fileDialogSelectionBox);
  return;
}								    /* end function ShowSaveAsFileDialog */



void SetImageSaveFormat(Widget		w,
			XtPointer	clientData,
			XtPointer	xt_call_data )
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if (fileFormat != (char*)NULL)				    /* Free up any space previously pointed to by this variable */
    free(fileFormat);
  fileFormat = strdup(clientData);				    /* Set a global determining file format for still images */
  return;
}								    /* end function SetImageSaveFormat */



void 
  SetOutputAnyAspectRatio (Widget w,
			   XtPointer client_data,
			   XtPointer xt_call_data)
{
  Position x;
  Position y;
  static XRectangle		widgetRect;
  Boolean			fourToThreeAspectRatio;
  int				originalWindowX;
  int				originalWindowY;
  int				result;
  Window			rootWindow;
  Window			child;
  Window			sourceWin;
  Dimension			originalWindowWidth;
  Dimension			originalWindowHeight;
  Dimension			borderWidth;
  int				channel;
  
  rootWindow = RootWindowOfScreen(XtScreen(outputWidget));
  XtVaGetValues(outputWidget,					    /* Get the current size of the video output widget */
		XtNwidth, &originalWindowWidth,
		XtNheight, &originalWindowHeight,
		XtNborderWidth, &borderWidth,
		XtNsourceWin, &sourceWin,
		XtNchannel, &channel,
		NULL);

  XTranslateCoordinates(XtDisplay(outputWidget), XtWindow(outputWidget),	    /* Get the absolute X & Y coordinates of the video... */
			rootWindow,				    /* ...output widget. */
			(int)-borderWidth, (int)-borderWidth,
			&originalWindowX, &originalWindowY,
			&child);
  widgetRect.x = (short)originalWindowX;
  widgetRect.y = (short)originalWindowY;
  widgetRect.width = (unsigned short)originalWindowWidth;
  widgetRect.height = (unsigned short)originalWindowHeight;

  XtVaSetValues(outputShell,
		XtNx, (Position)widgetRect.x,
		XtNy, (Position)widgetRect.y,
		NULL);
  XtVaSetValues(outputWidget,
		XtNnormalAspectRatio, False, 
		XtNx, (Position)widgetRect.x,
		XtNy, (Position)widgetRect.y,
		NULL);
  return;
}								    /* end function SetOutputAnyAspectRatio */



void 
  SetOutput4To3AspectRatio (Widget w,
			    XtPointer client_data,
			    XtPointer xt_call_data)
{
  static XRectangle		widgetRect;
  Boolean			fourToThreeAspectRatio;
  int				originalWindowX;
  int				originalWindowY;
  int				result;
  Window			rootWindow;
  Window			child;
  Window			sourceWin;
  Dimension			originalWindowWidth;
  Dimension			originalWindowHeight;
  Dimension			borderWidth;
  int				channel;
  
  rootWindow = RootWindowOfScreen(XtScreen(outputWidget));
  XtVaGetValues(outputWidget,					    /* Get the current size of the video output widget */
		XtNwidth, &originalWindowWidth,
		XtNheight, &originalWindowHeight,
		XtNborderWidth, &borderWidth,
		XtNsourceWin, &sourceWin,
		XtNchannel, &channel,
		NULL);

  XTranslateCoordinates(XtDisplay(outputWidget), XtWindow(outputWidget),	    /* Get the absolute X & Y coordinates of the video... */
			rootWindow,				    /* ...output widget. */
			(int)-borderWidth, (int)-borderWidth,
			&originalWindowX, &originalWindowY,
			&child);
  widgetRect.x = (short)originalWindowX;
  widgetRect.y = (short)originalWindowY;
  widgetRect.width = (unsigned short)originalWindowWidth;
  widgetRect.height = (unsigned short)originalWindowHeight;

  XtVaSetValues(outputShell,
		XtNx, (Position)widgetRect.x,
		XtNy, (Position)widgetRect.y,
		NULL);
  XtVaSetValues(outputWidget,
		XtNnormalAspectRatio, True, 
		NULL);
  
  XtVaSetValues(outputWidget,
		XtNx, (Position)widgetRect.x,
		XtNy, (Position)widgetRect.y,
		NULL);
  return;
}								    /* end function SetOutput4To3AspectRatio */



void ShowOutputRectangle(Widget		w,
			 XtPointer	client_data,
			 XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  static XRectangle		widgetRect;
  Boolean			fourToThreeAspectRatio;
  int				originalWindowX;
  int				originalWindowY;
  int				result;
  Window			rootWindow;
  Window			child;
  Window			sourceWin;
  Dimension			originalWindowWidth;
  Dimension			originalWindowHeight;
  Dimension			borderWidth;
  int				channel;
  
  rootWindow = RootWindowOfScreen(XtScreen(w));
  XtVaGetValues(outputWidget,					    /* Get the current size of the video output widget */
		XtNwidth, &originalWindowWidth,
		XtNheight, &originalWindowHeight,
		XtNborderWidth, &borderWidth,
		XtNsourceWin, &sourceWin,
		XtNchannel, &channel,
		NULL);

  XTranslateCoordinates(XtDisplay(w), XtWindow(outputWidget),	    /* Get the absolute X & Y coordinates of the video... */
			rootWindow,				    /* ...output widget. */
			(int)-borderWidth, (int)-borderWidth,
			&originalWindowX, &originalWindowY,
			&child);
  widgetRect.x = (short)originalWindowX;
  widgetRect.y = (short)originalWindowY;
  widgetRect.width = (unsigned short)originalWindowWidth;
  widgetRect.height = (unsigned short)originalWindowHeight;

  if(sourceWin != XtWindow(rootWindowWidget))			    /* If the window has changed (SelectWindow was used before this)... */
  {								    /* ...reset the aspect ratio, the source window, and reposition... */
    XtVaGetValues(output4To3AspectToggle,			    /* ...the output with respect to the root window. */
		  XmNset, &fourToThreeAspectRatio,
		  NULL);
    
    XtVaSetValues(outputWidget,
		  XtNchannel, (channel&0xffff0000)|PLX_IO_OFF,
		  XtNnormalAspectRatio, fourToThreeAspectRatio,
		  XtNsourceWin, XtWindow(rootWindowWidget),	    /* Pass the root window to disable window tracking. */
		  NULL);
    XtVaSetValues(outputShell,
		  XtNx, (Position)widgetRect.x,
		  XtNy, (Position)widgetRect.y,
		  NULL);
    XtVaSetValues(outputWidget,
		  XtNchannel, channel,
		  XtNx, (Position)widgetRect.x,
		  XtNy, (Position)widgetRect.y,
		  NULL);
  }
  rubberBandMode = MoveMode;					    /* Tell the app to move the output rectangle by default */
  DrawBox(w, rubberBandGC, widgetRect.x, widgetRect.y,		    /* Draw the new rubberband rectangle */
	  widgetRect.width, widgetRect.height);
  XWarpPointer(XtDisplay(w), None, rootWindow, 0, 0, 0, 0,	    /* Move the pointer to the origin of the video output widget */
	       (int)widgetRect.x, (int)widgetRect.y);
  result = XtGrabPointer(rootWindowWidget, False,			    /* Grab control of the pointer so that rubberbanding can... */
			 (PointerMotionMask |				    /* ...be done. */
			  ButtonPressMask |
			  ButtonReleaseMask),
			 GrabModeAsync, GrabModeAsync,
			 None, handCursor, CurrentTime);
  if(result !=GrabSuccess) 
  {
    switch(result)
    {
      case GrabNotViewable:
        printf("GrabNotViewable\n");
        break;
      case AlreadyGrabbed:
	printf("AlreadyGrabbed\n");
	break;
      case GrabFrozen:
	printf("GrabFrozen\n");
	break;
      case GrabInvalidTime:
	printf("GrabInvalidTime\n");
	break;
      case GrabSuccess:
	printf("GrabSuccess\n");
	break;
      }
  }
  
  XtAddRawEventHandler(rootWindowWidget,
		       (ButtonPressMask),
		       True, StartRubberBandAction, (XtPointer)&widgetRect);
  XtAddRawEventHandler(rootWindowWidget,
		       (PointerMotionMask),
		       True, MoveResize, (XtPointer)&widgetRect);
  return;
}								    /* end function ShowOutputRectangle */



void SelectOutputWindow (Widget		w,
			 XtPointer	client_data,
			 XtPointer	xt_call_data)
{
  int					channel;
  Window				theWindow;
  Boolean				isRootWindow;
  
  theWindow = SelectWindow(w, &isRootWindow);			    /* Select a window for showing output */
  XtVaSetValues(outputWidget,
		XtNnormalAspectRatio, False,
		NULL);
  XtVaGetValues(outputWidget,
		XtNchannel, &channel,
		NULL);
  XPlxSelectWindowHardwareInstance(XtDisplay(rootWindowWidget),
				   theWindow,
				   channel >> 16);
  XtVaSetValues(outputWidget,
		XtNchannel, (channel & 0xffff0000) | PLX_IO_OFF,
		XtNsourceWin, theWindow,
		NULL);
  if(isRootWindow)
  {
    XtVaSetValues(outputShell,
		  XtNx, (Position)0,
		  XtNy, (Position)0,
		  NULL);
    XtVaSetValues(outputWidget,
		  XtNx, (Position)0,
		  XtNy, (Position)0,
		  NULL);
  }
  XtVaSetValues(outputWidget,					    /* Tell the output widget to track the newly-selected window */
		XtNchannel, channel,
		NULL);
  return;
}								    /* end function SelectOutputWindow */



void MaximumSizeOutputWindow(Widget	w,
			     XtPointer	client_data,
			     XtPointer	xt_call_data)
{
  Window srcWin;
  Window unusedWindow;
  int	x;
  int	y;
  int	width;
  int	height;
  int	originalWindowX;
  int	originalWindowY;
  Dimension borderWidth;
  Boolean fourToThreeAspectRatio;
  
  XtVaGetValues(outputWidget,
		XtNborderWidth, &borderWidth,
		XtNsignalWidth, &width,
		XtNsignalHeight, &height,
		XtNsourceWin, &srcWin,
		NULL);
  XtVaGetValues(output4To3AspectToggle,
		XmNset, &fourToThreeAspectRatio,
		NULL);

  if(srcWin != XtWindow(rootWindowWidget))
  {
    /*
     * Maximum size does not apply when we
     * are tracking a window.
     */
    return;
  }
  XTranslateCoordinates(XtDisplay(w), XtWindow(outputWidget),	    /* Get the absolute X & Y coordinates of the video... */
			RootWindowOfScreen(XtScreen(w)),
			(int)borderWidth, (int)borderWidth,
			&originalWindowX, &originalWindowY,
			&unusedWindow);

  XtVaSetValues(outputShell,
		XtNx, (Position)originalWindowX,
		XtNy, (Position)originalWindowY,
		XtNwidth, (Dimension)width,
		XtNheight, (Dimension)height,
		NULL);
  
  XtVaSetValues(outputWidget,
		XtNsourceWin, RootWindowOfScreen(XtScreen(w)),	    /* Go back to whole-window mode for output */
		XtNnormalAspectRatio, fourToThreeAspectRatio,		    /* Reset to 4-to-3 aspect ratio in case window tracking...*/
		XtNx, (Position)originalWindowX,
		XtNy, (Position)originalWindowY,
		XtNwidth, (Dimension)width,
		XtNheight, (Dimension)height,
		NULL);						    /* ...had been on. */
  return;
}								    /* end function MaximumSizeOutputWindow */



void PopdownInput1Shell(Widget w,
			XtPointer client_data,
			XtPointer xt_call_data)
{
  XmString	offString = XmStringCreateSimple("Off");
  
  XmToggleButtonSetState(videoIn1Toggle, False, False);		    /* Manually turn off the videoIn1 toggle */
  XtVaSetValues(videoIn1Toggle, XmNlabelString, offString, NULL);   /* Set the On/Off string beside the toggle to "Off" */
  XtVaSetValues(stillInput1Button, XmNsensitive, False, NULL);	    /* Don't allow capture on the 1st video input window */
  XmStringFree(offString);
  return;
} /* end function PopdownInput1Shell */



void PopdownInput2Shell(Widget w,
			XtPointer client_data,
			XtPointer xt_call_data)
{
  XmString	offString = XmStringCreateSimple("Off");
  
  XmToggleButtonSetState(videoIn2Toggle, False, False);		    /* Manually turn off the videoIn2 toggle */
  XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);   /* Set the On/Off string beside the toggle to "Off" */
  XtVaSetValues(stillInput2Button, XmNsensitive, False, NULL);	    /* Don't allow capture on the 2nd video input window */
  XmStringFree(offString);
  return;
} /* end function PopdownInput2Shell */



void SetSignalStandard(Widget		w,
		       XtPointer	clientData,
		       XtPointer	xt_call_data)
{
  int		autoSearchOn;
  int		channel;
  int		signalStandard;
  Boolean	sync;
  signalSpec	theSignal;
  int		newChannel;
  int		currentChannel;
  Boolean	outputOn = False;
  int		input1Channel = 0;
  int		input2Channel = 0;
  int		outputChannel = 0;

  if(outputWidget != (Widget)NULL)
  {
    XtVaGetValues(videoOutToggle,
		  XmNset, &outputOn,
		  NULL);
    if(outputOn)
    {
      /*
       * Turn off output as we turn on input.
       */
      XmToggleButtonSetState(videoOutToggle, False, True);
      ProcessAllXtRequests();
    }
  }
  
  if ((XtPointer)0 == clientData)				    /* Was the "first available" special value passed in? */
  {								    /* Yes, do the appropriate thing */
    if ((input1Widget == selectedVideoWidget) &&		    /* Is the 1st widget being modified? */
	(input2Widget != (Widget)NULL))				    /* And, is the 2nd input widget already created? */
    {
      XtVaGetValues(input2Widget,				    /* Yes, see what channel the other video widget is using... */
		    XtNsync, &sync,				    /* ...and whether it has a valid sync signal detected. */
		    XtNchannel, &channel,
		    NULL);
    }
    else							    /* No, the 2nd widget is being modified; check on the 1st... */
    {								    /* ...widget's settings. */
      XtVaGetValues(input1Widget,				    /* See what channel the other video widget is using... */
		    XtNsync, &sync,				    /* ...and whether it has a valid sync signal detected. */
		    XtNchannel, &channel,
		    NULL);
    }
    if ((True == sync) && (PLX_INPUT_0 == channel) &&
	(input2Widget != (Widget)NULL))				    /* If the other widget sees a valid sync signal on the... */
    {								    /* ...1st digitizer, then set autoSearch to look for... */
      autoSearchOn = AutoSearchInput1;				    /* ...a signal on only the 2nd digitizer. */
      theSignal.s.channel = PLX_INPUT_1;
    }
    if (((True == sync) && (PLX_INPUT_1 == channel)) ||		    /* If the other widget sees a valid sync signal on the... */
	(input2Widget == (Widget)NULL))
    {								    /* ...2nd digitizer, then set autoSearch to look for... */
      autoSearchOn = AutoSearchInput0;				    /* ...a signal on only the 1st digitizer. */
      theSignal.s.channel = PLX_INPUT_0;
    }
    if (False == sync)						    /* If the other widget sees no valid sync, then allow... */
    {								    /* ...autoSearch to look for a signal on any digitizer. */
      autoSearchOn = True;
      theSignal.s.channel = PLX_INPUT_0;
    }
    XtVaSetValues(selectedVideoWidget,				    /* Let the widget find the 1st available valid input source */
		  XtNautoSearch, autoSearchOn,
		  NULL);
    theSignal.s.standard = PLX_NTSC;				    /* ...XVideo hardware. */
    theSignal.s.format = PLX_COMP;
    theSignal.s.instance = 0;
  }
  else								    /* No, an explicit source was requested; use the value... */
  {
    XtVaSetValues(selectedVideoWidget,
		  XtNautoSearch, False,
		  NULL);
    theSignal.i = (unsigned int)clientData;			    /* ...passed in. */
  }
  XtVaGetValues(selectedVideoWidget,
		XtNchannel, &currentChannel,
		NULL);
  XtVaSetValues(selectedVideoWidget,
		XtNchannel, (currentChannel & 0xffff0000) | PLX_IO_OFF,
		NULL);						    /* Disable video before changing channels */
  XtVaSetValues(selectedVideoWidget,				    /* Try to set signal standard to the newly-requested value */
		XtNchannel, PlxInputToChannel[theSignal.s.channel],
		XtNsignalStandard, theSignal.s.standard,
		XtNsignalFormat, theSignal.s.format,
		XtNinstance, theSignal.s.instance,
		NULL);
  XtVaGetValues(selectedVideoWidget,				    /* Find out if the signal standard really changed */
		XtNsignalStandard, &signalStandard,
		XtNsignalFormat, &inputSignalFormat,
		XtNautoSearch, &autoSearchOn,
		NULL);
  ResizeVideoWidgetIfNecessary(selectedVideoWidget);		    /* If signal std. changed, widget might need resizing */
  ShowInputSettings(selectedSettingsWidget,			    /* Update labels on the input settings panel */
		    (XtPointer)NULL, (XtPointer)theSignal.i);
  if (autoSearchOn != False)					    /* Is auto-search enabled? */
  {
    XtVaSetValues(selectedVideoWidget,				    /* Yes, disable it */
		  XtNautoSearch, False,
		  NULL);
  }

  ProcessAllXtRequests();
  if(selectedVideoWidget == input1Widget)
  {
    XtVaGetValues(input1Widget,
		  XtNchannel, &currentInput1Channel,
		  NULL);
  }
  else
  {
    XtVaGetValues(input2Widget,
		  XtNchannel, &currentInput2Channel,
		  NULL);
  }    
  /*
   * Now, see if it is safe to turn on output again.
   */
  if(input1Widget != (Widget)NULL)
  {
    XtVaGetValues(input1Widget,
		  XtNchannel, &input1Channel,
		  NULL);
  }
  if(input2Widget != (Widget)NULL)
  {
    XtVaGetValues(input2Widget,
		  XtNchannel, &input2Channel,
		  NULL);
  }
  if(outputOn)
  {
    /*
     * Can we safely turn output back on?  The rule is that
     * output cannot be used at the same time as the second
     * digitizer.  The following code takes into account
     * multiple boards.
     */
    outputChannel = PlxOutputToChannel[defaultOutputChannel];
    if((((input1Channel>>16) == (outputChannel>>16)) &&
       ((input1Channel & 0x0000ffff) == PLX_INPUT_1)) ||
       (((input2Channel>>16) == (outputChannel>>16)) &&
	((input2Channel & 0x0000ffff) == PLX_INPUT_1)))
    {
      /*
       * It is not safe to turn output on.
       */
    }
    else
    {
      /*
       * It is safe to turn output on.
       */
      XmToggleButtonSetState(videoOutToggle, True, True);
    }
  }    
  return;
} /* end function SetSignalStandard */



void SetOutputSignalType(Widget		w,
			 XtPointer	clientData,
			 XtPointer	xt_call_data)
{
  signalSpec	theSignal;
  
  theSignal.i = (unsigned int)clientData;
  defaultOutputChannel = theSignal.s.channel;

  XtVaSetValues(outputWidget,					    /* Try to set signal standard to the newly-requested value */
		XtNchannel, PlxOutputToChannel[theSignal.s.channel],
		XtNsignalStandard, theSignal.s.standard,
		XtNsignalFormat, theSignal.s.format,
		XtNinstance, theSignal.s.instance,
		NULL);
  PlxShowOutputSelections(outputMenuCascade, theSignal);	    /* Update the output settings cascade menu */
  return;
} /* end function SetOutputSignalType */



void WMDeleteWindowCallback(Widget	widget,
			    XtPointer	clientData,
			    XtPointer	callData)
{
  XmString	offString = XmStringCreateLocalized("Off");
  
  if (widget == input1Shell)
  {
    XmToggleButtonSetState(videoIn1Toggle, False, False);	    /* Turn off the 1st video input's toggle */
    XtVaSetValues(videoIn1Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(stillInput1Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 1st video input window */
  }
  if (widget == input2Shell)
  {
    XmToggleButtonSetState(videoIn2Toggle, False, False);	    /* Turn off the 2nd video input's toggle */
    XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(stillInput2Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 2nd video input window */
  }
  if (widget == mainWindow)					    /* Was the window manager used to quit the app instead of... */
  {								    /* ...the app's File menu "Quit" item? */
    Quit(widget, (XtPointer)NULL, (XtPointer)NULL);		    /* Yes, call the normal Quit callback anyway */
  }
  XmStringFree(offString);
  return;
} /* end function WMDeleteWindowCallback */



void ResizeVideoWidgetIfNecessary(Widget widget)
{
  int		signalStandard;
  Dimension	signalHeight;
  Dimension	signalWidth;
  Dimension	height;
  Dimension	width;
  
  XtVaGetValues(widget,						    /* Find out what the current widget's width and height are */
		XtNwidth, &width,
		XtNheight, &height,
		NULL);
  XtVaGetValues(widget,						    /* Find out what the widget's width & height *should* be... */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,			    /* ...if they fit their signal standards */
		NULL);
  if ((width != signalWidth) || (height != signalHeight))	    /* If the two values are in disagreement, resize the... */
  {								    /* ...video widget to fit the signal standard. */
    XtVaSetValues(widget,
		  XtNwidth, signalWidth,
		  XtNheight, signalHeight,
		  NULL);
  }
} /* end function ResizeWidgetIfNecessary */



void PopdownCallback(Widget w,
		     XtPointer client_data,
		     XtPointer xt_call_data)
{
  if (w == input1Shell)						    /* Disable the 1st input */
  {
    input1ShellPoppedUp = False;
    XtVaSetValues(input1ChannelButton, XmNsensitive, False, NULL);
    if (input2ShellPoppedUp == True)				    /* Is the other video input active? */
    {
      ShowInputSettings(input2ChannelButton, (XtPointer)NULL,	    /* Yes, display its current settings */
			(XtPointer)NULL);
    }
  }
  if (w == input2Shell)
  {
    input2ShellPoppedUp = False;
    XtVaSetValues(input2ChannelButton, XmNsensitive, False, NULL);
    if (input1ShellPoppedUp == True)				    /* Is the other video input active? */
    {
      ShowInputSettings(input1ChannelButton, (XtPointer)NULL,	    /* Yes, display its current settings */
			(XtPointer)NULL);
    }
  }
  if ((input1ShellPoppedUp == False) &&				    /* Are both input shells popped down? */
      (input2ShellPoppedUp == False))
  {								    /* Yes, don't allow saving of still images */
    XtVaSetValues(saveButton, XmNsensitive, False, NULL);
    XtVaSetValues(saveAsButton, XmNsensitive, False, NULL);
  }
  return;
} /* end function PopdownCallback */



void PopupCallback(Widget w,
		   XtPointer client_data,
		   XtPointer xt_call_data)
{
  if (w == input1Shell)
  {
    input1ShellPoppedUp = True;
    XtVaSetValues(input1ChannelButton, XmNsensitive, True, NULL);
    if (input2ShellPoppedUp == False)				    /* Is the other video input inactive? */
    {
      ShowInputSettings(input1ChannelButton, (XtPointer)NULL,	    /* Yes, it's okay to show this input's current settings */
			(XtPointer)NULL);			    /* ...(if the other input were active, there would be... */
    }								    /* ...no knowing if its input settings were being shown). */
    else
    {
      ShowInputSettings(input2ChannelButton, (XtPointer)NULL,	    /* Yes, it's okay to show this input's current settings */
			(XtPointer)NULL);			    /* ...(if the other input were active, there would be... */
    }
  }
  if (w == input2Shell)
  {
    input2ShellPoppedUp = True;
    XtVaSetValues(input2ChannelButton, XmNsensitive, True, NULL);
    if (input1ShellPoppedUp == False)				    /* Is the other video input inactive? */
    {
      ShowInputSettings(input2ChannelButton, (XtPointer)NULL,	    /* Yes, it's okay to show this input's current settings */
			(XtPointer)NULL);			    /* ...(if the other input were active, there would be... */
    }								    /* ...no knowing if its input settings were being shown). */
    else
    {
      ShowInputSettings(input1ChannelButton, (XtPointer)NULL,	    /* Yes, it's okay to show this input's current settings */
			(XtPointer)NULL);			    /* ...(if the other input were active, there would be... */
    }
  }
  if ((input1ShellPoppedUp == True) ||				    /* Is either input shell popped up? */
      (input2ShellPoppedUp == True))
  {								    /* Yes, allow saving of still images */
    XtVaSetValues(saveButton, XmNsensitive, True, NULL);
    XtVaSetValues(saveAsButton, XmNsensitive, True, NULL);
  }
  return;
} /* end function PopupCallback */



Boolean ZoomWorkProc(XtPointer	clientData)
{
  extern AppResources	vtResources;				    /* VideoTool-specific X Resources */
  Dimension	height;
  Dimension	width;
  int		x;
  int		y;
  int		signalFormat;
  int		outputSignalFormat;
  int		signalStandard;
  int		outputSignalStandard;
  int		channel;
  int		outputChannel;
  int		instance;
  int		newSelectedWidget;
  int		hue;
  int		saturation;
  int		contrast;
  int		brightness;
  int		currentInput;
  int		input1Channel;
  int		input2Channel;
  int		videoWidgetMap = 0;
  int		attachOutput = 0;
  Boolean	normalAspectRatio;
  Boolean	live;
  Boolean	cropped;
  XPlxCImage*	jpegData = (XPlxCImage*)NULL;
  Window	sourceWin;
  
  if(selectedVideoWidget == input1Widget)
  {
    currentInput = 0;
  }
  else
  {
    currentInput = 1;
  }
      /*
       * Be sure that the zoom factor is actually changing.
       */
  zoomFactor[currentInput] = XPlxGetZoomFactor(XtDisplay(selectedVideoWidget),
				 XtWindow(selectedVideoWidget));
  if(zoomFactor[currentInput] == (int)clientData)		    /* Zoom factor is not changing, exit. */
  {
    return(True);
  }
  if (input1Widget == selectedVideoWidget)			    /* Keep track of which video widget was active when this... */
    newSelectedWidget = 0;					    /* ...function was called. */
  else
    newSelectedWidget = 1;

  if((Widget)NULL != outputWidget)
  {
    XtVaGetValues(outputWidget,
		  XtNchannel, &outputChannel,
		  NULL);
    XtVaGetValues(selectedVideoWidget,
		  XtNchannel, &input1Channel,
		  NULL);
    if(outputChannel >> 16 ==
       input1Channel >> 16)
    {
      XtVaGetValues(outputWidget,				    /* Observe current widget values for later */
		    XtNchannel, &outputChannel,
		    XtNsignalFormat, &outputSignalFormat,
		    XtNsignalStandard, &outputSignalStandard,
		    XtNsourceWin, &sourceWin,
		    NULL);
      if(OutputAttachedToInputWidget(input1Widget))
	attachOutput = 1;
      if(OutputAttachedToInputWidget(input2Widget))
	attachOutput = 2;
      
      XtDestroyWidget(outputWidget);				    /* Yes, destroy it */
      ProcessAllXtRequests();
    }
  }      

  XPlxSetZoomFactor(XtDisplay(selectedVideoWidget),		    /* Tell the hardware to zoom to the requested zoom ratio */
		    XtWindow(selectedVideoWidget),
		    (int)clientData);
  ProcessAllXtRequests();
  
  zoomFactor[currentInput] = (int)clientData;			    /* Tell the application the new zoom ratio */
  XtVaGetValues(input1Widget,
		XtNchannel, &input1Channel,
		NULL);
  if(input2Widget != (Widget)NULL)
  {
    XtVaGetValues(input2Widget,
		  XtNchannel, &input2Channel,
		  NULL);
  }
  if((input1Channel >> 16) != 
     (input2Channel >> 16))
  {
    /*
     * Only one input window needs to be changed.
     */
    if(selectedVideoWidget == input1Widget)
    {
      videoWidgetMap |= 1;
    }
    if(selectedVideoWidget == input2Widget)
    {
      videoWidgetMap |= (1 << 1);
    }
  }
  else
  {
    /*
     * Both inputs are on the same video board.  The dimensions of both input
     * windows must be changed.
     */
    videoWidgetMap |= 3;
    zoomFactor[0] = zoomFactor[1] = zoomFactor[currentInput];
  }
  if(videoWidgetMap & (0x0001))
  {
    XtVaGetValues(input1Widget,					    /* Observe current widget values for later */
		  XtNchannel, &channel,
		  XtNinstance, &instance,
		  XtNsignalFormat, &signalFormat,
		  XtNsignalStandard, &signalStandard,
		  XtNhue, &hue,
		  XtNsaturation, &saturation,
		  XtNcontrast, &contrast,
		  XtNbrightness, &brightness,
		  XtNnormalAspectRatio, &normalAspectRatio,
		  XtNlive, &live,
		  XtNcropped, &cropped,
		  XtNwidth, &width,
		  XtNheight, &height,
		  NULL);
    XtDestroyWidget(input1Widget);				    /* Destroy the old input 1 widget */
    if(zoomFactor[currentInput] == 2)				    /* Modify the size based on the new zoomFactor. */
    {
      width *= 2;
      height *= 2;
    }
    else
    {
      width /= 2;
      height /= 2;
    }
    XtVaSetValues(input1Shell,
		  XtNwidth, width,
		  XtNheight, height,
		  NULL);
    input1Widget = (Widget)NULL;
    input1Widget =						    /* Create a video input widget for the first digitizer */
      XtVaCreateManagedWidget("input1Widget",
			      xVideoInWidgetClass,
			      input1Shell,
			      XtNchannel, channel,
			      XtNinstance, instance,
			      XtNsignalFormat, signalFormat,
			      XtNsignalStandard, signalStandard,
			      XtNautoSearch, False,
			      XtNhue, hue,
			      XtNsaturation, saturation,
			      XtNcontrast, contrast,
			      XtNbrightness, brightness,
			      XtNnormalAspectRatio, normalAspectRatio,
			      XtNlive, live,
			      XtNcropped, cropped,
			      XtNwidth, width,
			      XtNheight, height,
			      NULL);
    XtVaSetValues(input1Widget, XtNautoSearch, autoSearch, NULL);	    /* Restore current setting for "autoSearch" */
//    ProcessAllXtRequests();
  }
  if(videoWidgetMap & 0x0002)
  {
    if ((Widget)NULL != input2Widget)				    /* Was a 2nd input widget created? */
    {
      XtVaGetValues(input2Widget,					    /* Observe current widget values for later */
		    XtNchannel, &channel,
		    XtNinstance, &instance,
		    XtNsignalFormat, &signalFormat,
		    XtNsignalStandard, &signalStandard,
		    XtNhue, &hue,
		    XtNsaturation, &saturation,
		    XtNcontrast, &contrast,
		    XtNbrightness, &brightness,
		    XtNwidth, &width,
		    XtNheight, &height,
		    NULL);
      if(zoomFactor[currentInput] == 2)				    /* Modify the size based on the new zoomFactor. */
      {
	width *= 2;
	height *= 2;
      }
      else
      {
	width /= 2;
	height /= 2;
      }
      XtDestroyWidget(input2Widget);				    /* Yes, destroy it */
      XtVaSetValues(input2Shell,
		    XtNwidth, width,
		    XtNheight, height,
		    NULL);
      input2Widget = XtVaCreateManagedWidget("input2Widget",	    /* Re-create the 2nd input widget */
					     xVideoInWidgetClass,
					     input2Shell,
					     XtNchannel, channel,
					     XtNinstance, instance,
					     XtNsignalFormat, signalFormat,
					     XtNsignalStandard, signalStandard,
					     XtNautoSearch, False,
					     XtNhue, hue,
					     XtNsaturation, saturation,
					     XtNcontrast, contrast,
					     XtNbrightness, brightness,
					     XtNwidth, width,
					     XtNheight, height,
					     NULL);
      XtVaSetValues(input2Widget, XtNautoSearch, autoSearch, NULL);	    /* Restore current setting for "autoSearch" */
//      ProcessAllXtRequests();
    }
  }
  
  if (0 == newSelectedWidget)					    /* Restore the notion of which widget is currently selected */
    selectedVideoWidget = input1Widget;
  else
    selectedVideoWidget = input2Widget;

  if ((Widget)NULL != outputWidget)				    /* Was an output widget created (i.e., VIO board used)? */
  {								    /* Yes, re-create it */
    outputWidget = XtVaCreateManagedWidget("outputWidget",	    /* Re-create the output widget */
					   xVideoOutWidgetClass,
					   outputShell,
					   XtNchannel, (outputChannel & 0xFFFF0000) | PLX_IO_OFF,  /* Re-initialize output widget as off to prevent HP... */
					   XtNsignalFormat, outputSignalFormat, /* ...output hardware from "drifting"; that is,... */
					   XtNsignalStandard, outputSignalStandard, /* ...output signal does not seem to sync up... */
					   NULL);		    /* ...and so on an external monitor it slides horizontally. */
    switch(attachOutput)
    {
      case 0:
      {
	/*
	 * Output was not attached to one of the affected input windows.
	 * Select the same output window.
	 */
	XtVaSetValues(outputWidget,
		      XtNsourceWin, sourceWin,
		      NULL);
	break;
      }
      case 1:
      {
	/*
	 * Output was attached to the input1Widget.
	 */
	XPlxSelectWindowHardwareInstance(XtDisplay(rootWindowWidget),
					 GetShellWindow(XtWindow(input1Shell)),
					 outputChannel >> 16);
	XtVaSetValues(outputWidget,
		      XtNnormalAspectRatio, False,
		      XtNsourceWin, XtWindow(input1Shell),
		      NULL);
	break;
      }
      case 2:
      {
	/*
	 * Output was attached to the input2Widget.
	 */
	XPlxSelectWindowHardwareInstance(XtDisplay(rootWindowWidget),
					 GetShellWindow(XtWindow(input2Shell)),
					 outputChannel >> 16);
	XtVaSetValues(outputWidget,
		      XtNnormalAspectRatio, False,
		      XtNsourceWin, XtWindow(input2Shell),
		      NULL);
	break;
      }
    }      
    XtVaSetValues(outputWidget, XtNchannel, outputChannel, NULL);	    /* Now that widget is created, it's okay to enable output */
//    ProcessAllXtRequests();
  }
  XtVaGetValues(jpegWidget, XtNjpegData, &jpegData, NULL);	    /* Determine if the JPEG widget has image data to display */
  if (jpegData != (XPlxCImage*)NULL)				    /* Well, does it? */
  {								    /* Yes it does, resize it to account for new zoom ratio */
    displayDestRect.x = 0;
    displayDestRect.y = 0;
    displayDestRect.width = jpegData->width * zoomFactor[vtResources.board];
    displayDestRect.height = jpegData->height * zoomFactor[vtResources.board];
    XtUnmanageChild(jpegWidget);
    XtVaSetValues(imageShell,
		  XtNbaseWidth, (Dimension)(jpegData->width * zoomFactor[vtResources.board]),
		  XtNbaseHeight, (Dimension)(jpegData->height * zoomFactor[vtResources.board]),
		  XtNwidth, (Dimension)(jpegData->width * zoomFactor[vtResources.board]),
		  XtNheight, (Dimension)(jpegData->height * zoomFactor[vtResources.board]),
		  XtNmaxWidth, (Dimension)(jpegData->width * zoomFactor[vtResources.board]),
		  XtNmaxHeight, (Dimension)(jpegData->height * zoomFactor[vtResources.board]),
		  XtNminWidth, (Dimension)(jpegData->width * zoomFactor[vtResources.board]),
		  XtNminHeight, (Dimension)(jpegData->height * zoomFactor[vtResources.board]),
		  NULL);
    XtVaSetValues(jpegWidget,
		  XtNwidth, (Dimension)(jpegData->width * zoomFactor[vtResources.board]),
		  XtNheight, (Dimension)(jpegData->height * zoomFactor[vtResources.board]),
		  XtNdisplayDestRect, &displayDestRect,
		  NULL);
    XtManageChild(jpegWidget);
  }
  return(True);							    /* Done with the work proc; no need to call it any longer */
} /* end function ZoomWorkProc */



void ZoomCallback(Widget	w,
		  XtPointer	clientData,
		  XtPointer	xt_call_data)
{
  Boolean	buttonState;
  XtWorkProcId	zoomProcID;
  
  buttonState = XmToggleButtonGetState(w);
  if (False == buttonState)					    /* Don't bother doing anything if this toggle was turned off */
    return;
  zoomProcID = XtAppAddWorkProc(app_context,			    /* Start a work-proc to zoom the windows; Xt is... */
				ZoomWorkProc,			    /* ...poorly designed and this is the only way to assure... */
				(XtPointer)clientData);		    /* ...that widget resources will be freed when needed. */
  return;
} /* end function ZoomCallback */


void ChangeStillButtonLabel(Widget w, Boolean live)
{
  XmString	label;

  if(live)
  {
    label = XmStringCreateLocalized("Capture...");
    XtVaSetValues(w, XmNlabelString, label, NULL);   /* Change the appropriate label's string now */
    XmStringFree(label);					    /* Free space no longer needed for this temporary string */
    return;
  }
  else
  {
    label = XmStringCreateLocalized("Live");
    XtVaSetValues(w, XmNlabelString, label, NULL);	    /* Change the appropriate label's string now */
    XmStringFree(label);						    /* Free space no longer needed for this temporary string */
    return;

  }
  
}

void SaveStill(Widget		w,
	       XEvent*		event,
	       String*		params,
	       Cardinal*	numParams)
{
  Boolean	isLive;
  Widget	whichLabelWidget;
  
  if ((w != input1Widget) && (w != input2Widget))
  {
    return;
  }
  SelectVideoWidget(w);						    /* Make this widget the currently-selected widget */
  XtVaGetValues(w, XtNlive, &isLive, NULL);			    /* Determine whether this widget is live or not */
  if (w == input1Widget)					    /* Determine which "Capture.../Live" widget to update */
    whichLabelWidget = stillInput1Button;
  else
    whichLabelWidget = stillInput2Button;
  ChangeStillButtonLabel(whichLabelWidget, !isLive);
  if (False == isLive)						    /* If the video widget is stilled, then make it live again */
  {
    XtVaSetValues(w, XtNlive, True, NULL);
    XtUnmanageChild(fileDialogSelectionBox);
    ShowInputSettings(selectedSettingsWidget,			    /* Update the current widget's user interface settings */
		      (XtPointer)NULL, (XtPointer)NULL);
    return;
  }
  SetStillVideo((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);    /* Still the now-currently-selected widget */
  ShowSaveFileDialog((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL); /* Let a filename be chosen and the image saved */
  if (w == input1Widget)					    /* Try to raise the appropriate shell to the top so that... */
    XtPopup(input1Shell, XtGrabNone);				    /* ...images that are to be grabbed are not occluded by... */
  else								    /* ...other windows. */
    XtPopup(input2Shell, XtGrabNone);
  ShowInputSettings(selectedSettingsWidget,			    /* Update the current widget's user interface settings */
		    (XtPointer)NULL, (XtPointer)NULL);
  return;
} /* end function SaveStill */




void CaptureDelayCallback(Widget	w,
			  XtPointer	clientData,
			  XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(input1Widget, XtNstillDelay, callData->value, NULL); /* Change the value of 1st Widget's "stillDelay" resource */
  if ((Widget)NULL != input2Widget)				    /* Was the 2nd input widget created? */
    XtVaSetValues(input2Widget, XtNstillDelay, callData->value, NULL); /* Yes, change its "stillDelay" value, also */
  return;
} /* end function CaptureDelayCallback */




void CaptureCallback(Widget	w,
		     XtPointer	clientData,
		     XtPointer	xt_call_data)
{
  Widget	inputWidget;
  
  if (1 == (int)clientData)					    /* Determine which video source to capture/restart */
    inputWidget = input1Widget;
  else
    inputWidget = input2Widget;
  if ((Widget)NULL == inputWidget)				    /* Don't bother trying to do any work on an empty Widget */
    return;
  SaveStill(inputWidget, (XEvent*)NULL, (String*)NULL,		    /* Capture or restart the appropriate video source */
	    (Cardinal*)NULL);
  return;
} /* end function CaptureCallback */
