/*
 *   Copyright (c) 1996 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */
/* $Header: video.c,v 1.9 96/10/31 00:06:41 lovvik Exp $ */
/* $Log:	video.c,v $
 * Revision 1.9  96/10/31  00:06:41  00:06:41  lovvik (Paul Lovvik)
 *   Fixed a problem in Input2Callback().  This function was turning
 *   off the output toggle button without calling the function that
 *   handles the button.  It turns out that turning off the toggle
 *   button in that case was unnecessary anyway.  This change is
 *   specific to multi-board support.
 * 
 * Revision 1.8  1996/10/30  02:38:16  lovvik
 *   Improved multiboard support.  Specifically, I fixed some minor
 *   problems in OutputCallback() and Input2Callback().
 *
 * Revision 1.7  1996/10/15  16:08:19  lovvik
 *   Changed the way TIFF images were saved.  The output widget uses
 *   a work proc to track a window.  This interfered with the work proc
 *   that saved the TIFF image.  I simply removed the work proc and put
 *   the guts into SaveImageFileInFormat().
 *
 * Revision 1.6  1996/10/03  17:38:48  lovvik
 *   Fixed a minor memory error.
 *
 * Revision 1.5  1996/08/26  22:35:39  lovvik
 *   Modified Input2Callback.  If output is on and tracking the
 *   Input 2 window, output will turn off when the user turns off
 *   Input 2.
 *
 * Revision 1.4  1996/08/07  16:48:19  lovvik
 *   Removed RestoreInput2IfNeeded().  This was a hack meant to
 *   overcome a problem in Parallax libraries.  The library has
 *   now been fixed, so this is not necessary.
 *
 * Revision 1.3  1996/06/26  20:57:26  lovvik
 *   Fixed a minor compile error on HP.
 *
 * Revision 1.2  1996/06/26  16:10:16  lovvik
 *   Fixed OsSpecificInit() to correctly deal with a combination
 *   of Parallax framebuffer and overlay cards.
 *
 * Revision 1.1  1996/06/12  22:10:03  lovvik
 * Initial revision
 * */

static char videoRcsId[] = "$Header: video.c,v 1.9 96/10/31 00:06:41 lovvik Exp $";

#include "VideoTool.h"
#include <XPlxExt.h>
#include <XPlxProtos.h>


static Dimension	tiffImageWidth;				    /* Variables local to this module, for saving TIFF images */
static Dimension	tiffImageHeight;
static Window		tiffImageWindow;
static char		tiffImageFileName[MAXPATHLEN];
XPlxHardwareInfo 	hardwareInfo;
int			defaultOutputChannel;
int			currentInput1Channel;
int			currentInput2Channel;


void Input2Callback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data )
{
  static int			lastStandard = PLX_NTSC;
  int				signalStandard;
  int				channel;
  int				input1HardwareInstance;
  int				input2HardwareInstance;
  Boolean			outputOn;
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)					    /* Should input 2 be activated? */
  {								    /* Yes, do the appropriate book work */
    if(((currentInput2Channel & 0x0000ffff) == PLX_INPUT_1) &&
       ((currentInput2Channel >> 16) == (PlxOutputToChannel[defaultOutputChannel]>>16)))
    {
      /*
       * Input 2 and output will conflict.
       */
      XtVaGetValues(videoOutToggle,
		    XmNset, &outputOn,
		    NULL);
      if(outputWidget != (Widget)NULL && outputOn)
      {
	/*
	 * Output is on, so we must turn it off.
	 */
	XmToggleButtonSetState(videoOutToggle, False, True);
	ProcessAllXtRequests();
      }
    }
  
    XtVaSetValues(input2Widget, XtNchannel,
		  currentInput2Channel,				    /* Restart video on the 2nd input */
		  XtNlive, True,				    /* Hack: force video to be live even if it was still... */
		  NULL);					    /* ...since still video allows output to steal the Port. */
    XtVaGetValues(input2Widget,					    /* Determine if an active signal is on input 2 and... */
		  XtNsync, &input2Sync,				    /* ...the new signal standard for the widget, if any. */
		  XtNsignalStandard, &signalStandard,
		  NULL);
    ShowInputSettings(input2ChannelButton, (XtPointer)NULL,	    /* Update the current input settings now that a new... */
		      (XtPointer)NULL);				    /* ...input signal has been selected. */

    if (lastStandard != signalStandard)				    /* Did the signal standard for this widget change? */
      ResizeVideoWidgetIfNecessary(input2Widget);		    /* Yes, widget might need resizing */
    lastStandard = signalStandard;
    XtPopup(input2Shell, XtGrabNone);				    /* Popup the 2nd video input's shell */
    XtVaSetValues(videoIn2Toggle, XmNlabelString, onString, NULL);
    XtVaSetValues(stillInput2Button, XmNsensitive, True, NULL);	    /* Allow capture on the 2nd video input window */
  }
  else								    /* Input 2 should not be activated, disable input 2 for now */
  {
    if(OutputAttachedToInputWidget(input2Widget))
    {
      /*
       * Need to turn off output.
       */
      XmToggleButtonSetState(videoOutToggle, False, True);
    }
    XtPopdown(input2Shell);
    XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(stillInput2Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 2nd video input window */
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function Input2Callback */



void 
  OutputCallback(Widget		w,
		 XtPointer	client_data,
		 XtPointer	xt_call_data)
{
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  int				channel;
  int				instance = 0;
  int				input1HardwareInstance;
  int				input2HardwareInstance;
  XmString			label;
  int				signalFormat;
  int				signalStandard;
  signalSpec			theSignal;
  Position			x;
  Position			y;
  Dimension			width;
  Dimension			height;
  Window			outputSourceWindow;
  int				outputChannel;
  
  if (callData->set == True)					    /* Was output requested to be turned on? */
  {								    /* Yes, do the necessary bookkeeping */
    outputChannel = PlxOutputToChannel[defaultOutputChannel];
    XtVaGetValues(input1Widget,
		  XtNchannel, &input1HardwareInstance,
		  NULL);
    XtVaGetValues(input2Widget,
		  XtNchannel, &input2HardwareInstance,
		  NULL);
    if((input1HardwareInstance & 0xffff0000) == (outputChannel & 0xffff0000) ||
       (input2HardwareInstance & 0xffff0000) == (outputChannel & 0xffff0000))

    {
      /*
       * The output is going to conflict with either Input 1
       * or Input 2.  I will turn off the conflicting video
       * window.
       */
      if(((input1HardwareInstance & 0xffff) == PLX_INPUT_1) &&
	 ((input1HardwareInstance & 0xffff0000) == (outputChannel & 0xffff0000)))
      {
	XtPopdown(input1Shell);					    /* Remove the 1st input from the screen for now */
	XmToggleButtonSetState(videoIn1Toggle, False, False);	    /* Turn off the 1st video input's toggle */
	XtVaSetValues(videoIn1Toggle, XmNlabelString, offString, NULL);
	XtVaSetValues(stillInput1Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 1st video input window */
	XtVaGetValues(input1Widget,
		      XtNchannel, &channel,				    /* Get the current channel.  The upper 16 bits ... */
		      NULL);					    /* ... represent the hardware instance. */
	XtVaSetValues(input1Widget, XtNchannel,
		      PLX_IO_OFF | (channel & 0xffff0000),
		      NULL);					    /* Turn off the 1st input so as not to conflict with output */
	ProcessAllXtRequests();
      }
      if(((input2HardwareInstance & 0xffff) == PLX_INPUT_1) &&
	 ((input2HardwareInstance & 0xffff0000) == (outputChannel & 0xffff0000)))
      {
	XtPopdown(input2Shell);					    /* Remove the 2nd input from the screen for now */
	XmToggleButtonSetState(videoIn2Toggle, False, False);	    /* Turn off the 2nd video input's toggle */
	XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
	XtVaSetValues(stillInput2Button, XmNsensitive, False, NULL);    /* Don't allow capture on the 2nd video input window */
	XtVaGetValues(input2Widget,
		      XtNchannel, &channel,				    /* Get the current channel.  The upper 16 bits ... */
		      NULL);					    /* ... represent the hardware instance. */
	XtVaSetValues(input2Widget, XtNchannel,
		      PLX_IO_OFF | (channel & 0xffff0000),
		      NULL);					    /* Turn off the 2nd input so as not to conflict with output */
	ProcessAllXtRequests();
      }
    }
    
    XtVaGetValues(outputShell,
		  XtNx, &x,
		  XtNy, &y,
		  NULL);
    XtVaGetValues(outputWidget,
		  XtNwidth, &width,
		  XtNheight, &height,
		  XtNsourceWin, &outputSourceWindow,
		  NULL);
    XPlxSelectWindowHardwareInstance(XtDisplay(rootWindowWidget),
				     outputSourceWindow,
				     PlxOutputToChannel[defaultOutputChannel] >> 16);
    XtVaSetValues(outputWidget,
		  XtNx, x,
		  XtNy, y,
		  XtNchannel, PlxOutputToChannel[defaultOutputChannel],
		  NULL);    /* Turn on generation of a video signal, if not already on */
    XtVaSetValues(videoOutToggle, XmNlabelString, onString, NULL);
    EnableOutputControls();
    XtVaGetValues(outputWidget,					    /* Try to get the currently-selected output values */
		  XtNchannel, &channel,
		  XtNsignalStandard, &signalStandard,
		  XtNsignalFormat, &signalFormat,
		  NULL);
    theSignal.s.format = signalFormat;
    theSignal.s.instance = instance;
    theSignal.s.standard = signalStandard;
    theSignal.s.channel = channel;
    PlxShowOutputSelections(outputMenuCascade, theSignal);	    /* Update the output settings cascade menu */
  }
  else								    /* No, output was requested to be turned off */
  {
    XtVaGetValues(outputWidget,
		  XtNchannel, &channel,				    /* Get the current output channel.  The upper ... */
		  NULL);					    /* ...16 bits represent the hardware instance. */
    XtVaSetValues(outputWidget,
		  XtNchannel, (channel & 0xFFFF0000) | PLX_IO_OFF,
		  NULL);	    /* Turn off generation of a video signal */
    XtVaSetValues(videoOutToggle, XmNlabelString, offString, NULL);
    DisableOutputControls();
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function OutputCallback */



void OsSpecificInit(void)
{
  extern		AppResources	vtResources;		    /* VideoTool-specific X Resources */
  Display		*display;
  Window		rootWindow;
  GC			gc;
  plx_IO*		plxIO;
  XPlxHardwareInfo 	hwInfo;
  XPlxHardwareInfo 	tmpHWInfo;
  int			hwCount;
  int			index;
  
  autoSearch = True;
  display = XtDisplay(mainWindow);				    /* Get hardware information. */
  rootWindow = RootWindowOfScreen(XtScreen(mainWindow));
  gc = XCreateGC(display, rootWindow, 0, 0);

  /*
   * Find out if the hardware is exclusively overlay.
   * If not, the overlay field will be set to 0.  This
   * way, I know whether or not to destroy unneeded
   * interface widgets.
   */
  hwInfo.allBits = 0;
  hwInfo.info.overlay = 1;
  hwCount = XPlxGetHardwareInstances(display);
  for(index = 0; index < hwCount; index++)
  {
    XPlxSetDrawableToAdaptor(display, rootWindow, index);
    tmpHWInfo = XPlxQueryHardware(display, rootWindow, gc);
    hwInfo.info.overlay &= tmpHWInfo.info.overlay;
    /*
     * Only see if there is compression on the default board.
     * (This is the board that saved/opened images will use,
     * and can be set using the -board option).
     */
    if(index == vtResources.board)
      hwInfo.info.numJPEGCompressors = tmpHWInfo.info.numJPEGCompressors;
  }
  
  plxIO = XPlxQueryConfig(display,
			  rootWindow,
			  gc);
  XFreeGC(display, gc);

  if(plxIO != NULL)
  {
    hardwareInfo.info.overlay = plxIO->overlay;
    hardwareInfo.info.zoom = plxIO->zoom;
    hardwareInfo.info.numJPEGCompressors = plxIO->numJPEGCompressors;
    hardwareInfo.info.numMPEGDecoders = plxIO->numMPEGDecoders;
    hardwareInfo.info.numH320Compressors = plxIO->numH320Compressors;
    hardwareInfo.info.reserved = plxIO->reserved;
    hardwareInfo.info.hardware = plxIO->hardware;
  }

  if(hwInfo.info.overlay == 1)					    /* Is this an overlay board? */
  {
    /*
     * If only overlay boards are being used, unmap
     * widgets that do not work with overlay.
     */
    XtDestroyWidget(effectsOptionButton);			    /* Until NVRAM works, don't let users save or load... */
    XtDestroyWidget(effectsOptionLabel);			    /* ...color settings on overlay boards. */
    XtDestroyWidget(loadSettingsCascade);
    XtDestroyWidget(saveSettingsCascade);
    effectsOptionButton = effectsOptionLabel =
      loadSettingsCascade = saveSettingsCascade =
	(Widget)NULL;
    XtVaSetValues(aspectRatioLabel,				    /* Reposition the rest of the widgets to take up the... */
		  XmNtopOffset, 5,				    /* ...space left by the effects option button. */
		  XmNtopWidget, inputTypeInfoLabel,
		  XmNtopAttachment, XmATTACH_WIDGET,
		  NULL);
  }

  if(hwInfo.info.numJPEGCompressors == 0)			    /* If the hardware has no compression... */
  {
    /*
     * If no JPEG compressors are available,
     * do not allow the user to save using JPEG
     * compression.
     */
    XtUnmanageChild(saveAsJFIFButton);				    /* ...don't let users save using JPEG compression. */
    XtUnmanageChild(saveAsJPEGButton);

    /* 
     * Also, do not let the user open an image.
     */
    XtUnmanageChild(openButton);
  }
  defaultOutputChannel = PlxChannelToOutput((vtResources.board<<16)|PLX_OUTPUT_0);
  currentInput1Channel = PLX_INPUT_0;
  currentInput2Channel = PLX_INPUT_1;
  if(input1Widget != (Widget)NULL)
  {
    XtVaGetValues(input1Widget,
		  XtNchannel, &currentInput1Channel,
		  NULL);
  }
  if(input2Widget != (Widget)NULL)
  {
    XtVaGetValues(input2Widget,
		  XtNchannel, &currentInput2Channel,
		  NULL);
  }
  return;
} /* end function OsSpecificInit */



void SaveImageFileInFormat(Display*	display,
			   Window	imageWindow,
			   Dimension	width,
			   Dimension	height,
			   char*	fileFormat,
			   char*	fileName)
{
  char		convertCommand[32 + MAXPATHLEN];
  char*		tempFileName = "/tmp/.VideoToolTempImage";
  FILE*		imageFile;
  XImage*	localImage;
  int		result;
  char		windowName[256];
  
  if (strcmp(fileFormat, "SUN") == 0)				    /* Hack!!!!!: if the image type to be saved is Sunraster,... */
  {								    /* ...go around normal means on the HP platform to save... */
    XtUnmanageChild(fileDialogSelectionBox);
    localImage = (XImage*)XPlxGetImage(display, imageWindow, 0, 0,
				       width, height,
				       AllPlanes, ZPixmap);
    imageFile = fopen(fileName, "w");
    if (imageFile == (FILE*)NULL)
    {
      sprintf(errorMessage, "Could not open the file %s for writing.\n", fileName);
      PrintErrorMessage(errorMessage);
      return;
    }
    pr_dump_ximage(imageFile, localImage, 0, NULL, NULL, NULL);
    fclose(imageFile);						    /* Make sure all data are written out, then close the file */
    XDestroyImage(localImage);
  }
  if (strcmp(fileFormat, "TIFF") == 0)				    /* Is the user requesting to save as a TIFF image? */
  {
    tiffImageWindow = imageWindow;
    tiffImageHeight = height;
    tiffImageWidth = width;
    strcpy(tiffImageFileName, fileName);
    XtUnmanageChild(fileDialogSelectionBox);
    ProcessAllXtRequests();

    localImage = (XImage*)XPlxGetImage(XtDisplay(selectedVideoWidget),
				       tiffImageWindow, 0, 0,
				       tiffImageWidth, tiffImageHeight,
				       AllPlanes, ZPixmap);
    imageFile = fopen(tempFileName, "w");
    if (imageFile == (FILE*)NULL)
    {
      sprintf(errorMessage, "Could not create a temporary file %s for saving your image.\n");
      PrintErrorMessage(errorMessage);
      XDestroyImage(localImage);
      return;
    }
    pr_dump_ximage(imageFile, localImage, 0, NULL, NULL, NULL);
    fclose(imageFile);						    /* Make sure all data are written out, then close the file */
    XDestroyImage(localImage);
    sprintf(convertCommand,					    /* Yes, create a command to convert from the Sun raster... */
	    "convert -comment 'Created by Parallax Graphics, Inc. VideoTool' +matte %s tiff:%s",
	    tempFileName,						    /* ...image into a TIFF using ImageMagick tools. */
	    tiffImageFileName);
    result = system(convertCommand);
    if (result == -1)						    /* Did the conversion fail? */
    {								    /* Yes, print an error message and return */
      sprintf(errorMessage, "Could not create the TIFF image %s.\n", tiffImageFileName);
      PrintErrorMessage(errorMessage);
    }
    unlink(tempFileName);						    /* Remove the temporary Sun Raster file */
    return;
  } /* end if TIFF is the requested file format */
  return;
}								    /* end function SaveImageFileInFormat */
