/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* This program creates a video input widget and a JPEG widget.  The purpose of
 * the program is to show how to grab compressed images from a video source.
 * The program can take an optional argument, that being the ID of any X window.
 * If this optional argument is specified, the program will use that window
 * as the window from which compressed data should be created (i.e., the JPEG
 * window will digitize and compress the contents of that window) instead of
 * using the video input widget's window.
 *
 * Usage: jpegTest <optionalWindowID>
 */
/* $Header: /plx/source/Xclients/generic/RCS/jpegTest.c,v 1.13 1996/05/09 18:27:22 lovvik Exp $ */
/* $Log: jpegTest.c,v $
 * Revision 1.13  1996/05/09  18:27:22  lovvik
 *   Now works in zoom mode.
 *
 * Revision 1.12  1995/06/27  22:31:45  drapeau
 * Minor fix; removed extraneous diagnostic message.
 *
 * Revision 1.11  1993/11/17  02:30:34  drapeau
 * Restored live video, as it should be.
 *
 * Revision 1.10  93/09/10  12:37:33  drapeau
 * First public version of this test program.
 *  */
static char jpegTestRcsId[] = "$Header: /plx/source/Xclients/generic/RCS/jpegTest.c,v 1.13 1996/05/09 18:27:22 lovvik Exp $";

#include <stdio.h>
#include <sys/param.h>
#include <signal.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <XPlxExt.h>
#include <XVideoIn.h>
#include <JPEG.h>

static XrmOptionDescRec options[] = {
	{"-channel",		"*videoIn.channel",			XrmoptionSepArg,	NULL, },
	{"-sourceWin",		"*jpeg.sourceWin",			XrmoptionSepArg,	NULL, },
};

Widget top, videoShell, jpegShell, videoWidget, jpegWidget;
int		zoomFactor;
Dimension	width;
Dimension	height;

void main(int argc, char** argv)
{
  XtAppContext	appContext;
  Window	windowID;
  int		theWindow = 0;
  static XRectangle	compressSourceRect;
  static XRectangle	compressDestRect;
  static XRectangle	displaySourceRect;
  static XRectangle	displayDestRect;
  
  top = XtAppInitialize(&appContext, "JPEGTest", options, XtNumber(options),
			&argc, argv,
			NULL, (ArgList)NULL, 0);
  if (argc > 1)
  {
    printf("Argument was specified: ::%s::.\n", argv[1]);
    theWindow = (int)atoi(argv[1]);
  }
  videoWidget = XtVaCreateManagedWidget("videoIn",		    /* Create a VideoIn widget, used for displaying... */
					xVideoInWidgetClass,	    /* ...digitized video on-screen. */
					top,
					XtNlive, True,		    /* Make sure the video is still, not running live */
					NULL);
  XtRealizeWidget(top);
  zoomFactor = XPlxGetZoomFactor(XtDisplay(videoWidget),
				 XtWindow(videoWidget));

  XtVaGetValues(videoWidget,
		XtNwidth, &width,
		XtNheight, &height,
		NULL);
  width -= width % (16 * zoomFactor);
  height -= height % (16 * zoomFactor);
  XtVaSetValues(videoWidget,
		XtNwidth, width,
		XtNheight, height,
		NULL);
  jpegShell = XtVaCreateManagedWidget ("jpegShell",		    /* Create a shell to hold the JPEG compression widget */
				       applicationShellWidgetClass, 
				       top,
				       XtNwidth, width,
				       XtNheight, height,
				       NULL);
  if (theWindow != 0)						    /* Determine which window to point to: the JPEG widget... */
    windowID = (Window)theWindow;				    /* points to a window and uses that as the source for... */
  else								    /* ...compressing image data */
    windowID = XtWindow(videoWidget);

  compressSourceRect.x = 0;
  compressSourceRect.y = 0;
  compressSourceRect.width = width;
  compressSourceRect.height = height;

  compressDestRect.x = 0;
  compressDestRect.y = 0;
  compressDestRect.width = width;
  compressDestRect.height = height;

  displaySourceRect.x = 0;
  displaySourceRect.y = 0;
  displaySourceRect.width = width/zoomFactor;
  displaySourceRect.height = height/zoomFactor;
  
  displayDestRect.x = 0;
  displayDestRect.y = 0;
  displayDestRect.width = width;
  displayDestRect.height = height;

  jpegWidget = XtVaCreateManagedWidget("jpeg",			    /* Create a JPEG widget, used for compressing images and... */
				       jpegWidgetClass,		    /* ...displaying compressed images */
				       jpegShell,
				       XtNsourceWin, windowID,	    /* Tell the JPEG widget where to get fresh image data */
				       XtNwidth, width,
				       XtNheight, height,
				       XtNcompressSourceRect,
				       &compressSourceRect,
				       XtNcompressDestRect,
				       &compressDestRect,
				       XtNdisplaySourceRect,
				       &displaySourceRect,
				       XtNdisplayDestRect,
				       &displayDestRect,
				       NULL);
  XtRealizeWidget(jpegShell);
  XtAppMainLoop(appContext);
}								    /* end function main */
