/*
 * xvshow.c:    Test program for Xv video extension access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     26 May 93 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:    xvshow.c,v $
 * Revision 1.4  95/07/24  10:51:00  10:51:00  dist ()
 * Andy Goodrich: Freeze for real HP 9.1 release.
 * 
 * Revision 1.3  95/06/21  17:17:38  17:17:38  dist ()
 * Andy Goodrich: Added support for viewing multiple images from a file.
 * 
 * Revision 1.2  95/06/07  16:05:19  16:05:19  dist ()
 * Andy Goodrich: Correct scaling problems introduced when doing some 
 *                testing for JJ.
 * 
 * Revision 1.1  95/05/16  10:10:37  10:10:37  dist ()
 * Initial revision
 * 
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 */
#include <sys/types.h>          /* Standard types for this C environment. */
#include <stdio.h>              /* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "O_XV.h"
#include "rasterfile.h"

/*  FORWARD REFERENCES: */

XImage* read_sun_raster();

/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for system for this application.
 |
 *----------------------------------------------------------------------------*/
main( argc, argv )
    int     argc;   /* = # of command arguments parsed by shell. */
    char*   argv[]; /* = vector of pointers to command arguments parsed. */
{
    Atom        a_wid;      /* Atom for getting wid of port 0. */
    Atom        a_ximage;   /* Atom designating an XImage. */
    int         dump_hdr;   /* = 1 if should dump raster file header. */
    XEvent      event;      /* Information about X event we intercepted. */
    FILE*       fd;         /* File descriptor to read image from. */
    int         gov;        /* = 1 if should use gov mode. */
    int         height;     /* Height of image to display. */
    int         i_argv;     /* # of element in 'argv' accessing. */
    int         i_file;     /* Index into argv of name of file to open. */
    int         i_pixel;    /* Pixel in p_image->data now accessing. */
    int         output;     /* = 1 if should output to tv. */
    Display*    p_display;  /* Display accessing. */
    XImage*     p_image;    /* X image to display. */
    O_XV*       p_xv;       /* Xv access object. */ 
    O_XV*       p_xv1;      /* Xv access object. */ 
    int         width;      /* Width of image to display. */
    int         widX;       /* Window id for port other than 0. */
    int         wid0;       /* Window id for port 0. */
    int			image_len;	/* length in bytes of image */
    

    /*  PROCESS OUR COMMAND PARAMETERS: 
     *
     *    An unrecognized keyword is assumed to be the file to be displayed.
     */
    dump_hdr = 0;
    gov = 0;
    output = 0;
    i_file = 0;
    for ( i_argv = 1; i_argv < argc; i_argv++ )
    {
        if ( !strcmp(argv[i_argv], "-dump") )
        {
            dump_hdr = 1;
        }
        else if ( !strcmp(argv[i_argv], "-gov") )
        {
            gov = 1;
        }
        else if ( !strcmp(argv[i_argv], "-output") )
        {
            output = 1;
        }
        else
        {
            i_file = i_argv;
        }
    }
    if ( i_file == 0 )
    {
        fprintf(stderr, "Usage: xvshow [-dump] [-gov] [-output] file\n");
        return 0;
    }


    /*  OPEN THE X DISPLAY AND CREATE AN XV ACCESS OBJECT FOR PORT 0. */

    if ( !(p_display = XOpenDisplay(0)) ) 
    {
        fprintf(stderr, "\nCould not open display\n");
        return 0;
    }

    if ( !(p_xv = O_XV_O_XV( 0, p_display, 0, 0 )) )
    {
        fprintf(stderr, "\nCould create O_XV object\n");
        return 0;
    }


    /*  OPEN THE FILE TO BE DISPLAYED AND READ ITS CONTENTS INTO AN XIMAGE. */

    if ( !(fd = fopen(argv[i_file], "r")) )
    {
        fprintf(stderr, "*** Unable to open file %s\n", argv[i_file]);
        return 0;
    }
    p_image = read_sun_raster(fd, dump_hdr, p_display, &width, &height);
    if ( !p_image )
    {
        fprintf(stderr, 
            "Unable to parse a sun raster file in %s\n", argv[i_file]);
        return 0;
    }
    fprintf(stderr, "Outputing image of size %d by %d\n", width, height);
    O_XV_Window( p_xv, -1, -1 );
    p_xv->win_width = width;
    p_xv->win_height = height;
    image_len = (width * height) * sizeof(unsigned int);

    /*  GOV MODE: USE PLX FORMAT & SET THE WINDOW IDS OF ALL PIXELS IN THE IMAGE
     *
     *    For non-zero pixels use widX, for zero pixels use wid0
     */
    if ( gov ) 
    {
        a_ximage = XInternAtom(p_display,"XV_IMAGE_PLX_XIMAGE32",False);
        a_wid = XInternAtom(p_display,"XV_PLX_WID",False);
        if ( O_XV_Get_Attribute(p_xv, a_wid, &wid0) )
        {
            fprintf(stderr, "Bad return code from getting XV_PLX_WID\n");
            return 0;
        }
    
        widX = (wid0) ? 0 : 0x0e;

        for ( i_pixel = 0; i_pixel < width*height; i_pixel++ )
        {
            if ( ((int*)(p_image->data))[i_pixel] )
            {
                ((int*)(p_image->data))[i_pixel] |= (widX << 24);
            }
            else
            {
                ((int*)(p_image->data))[i_pixel] |= (wid0 << 24);
            }
        }
        O_XV_Video_On(p_xv); 
    }
    
    /* NON-GOV MODE: USE A NORMAL XIMAGE REQUEST. */

    else
    {
/* Hack! */
    widX = 0;
    wid0 = 0;
    
        for ( i_pixel = 0; i_pixel < width*height; i_pixel++ )
        {
            if ( ((int*)(p_image->data))[i_pixel] )
            {
                ((int*)(p_image->data))[i_pixel] |= (widX << 24);
            }
            else
            {
                ((int*)(p_image->data))[i_pixel] |= (wid0 << 24);
            }
        }
        O_XV_Video_On(p_xv); 
/* End hack */
        a_ximage = XInternAtom(p_display,"XV_IMAGE_XIMAGE",False);
        printf("a_ximage = %d\n", a_ximage);
    }


    /*  SEND THE IMAGE TO THE DISPLAY. */

    XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, a_ximage,
        image_len, p_image, 0, 0, width, height, 0, 0, width, height);


    /*  IF OUTPUT IS REQUESTED CREATE A SECOND PORT OBJECT AND USE IT. */

    if ( output )
    {
        if ( !(p_xv1 = O_XV_O_XV( 0, p_display, 0, 1 )) )
        {
            fprintf(stderr, "\nCould create O_XV object\n");
            return 0;
        }
        O_XV_Source( p_xv1, 0, XV_NTSC, XV_COMPOSITE, 0 );
        XvGetVideo(p_display, p_xv1->port_id, p_xv->w_main, p_xv->gc,
            0, 0, p_xv->sig_width, p_xv->sig_height,
            0, 0, p_xv->sig_width, p_xv->sig_height);
    }


    /*  LOOP PROCESSING EVENTS GENERATED BY THE OPERATOR OR WINDOWING SYSTEM. */

#ifndef NOTDEF
    XSetGraphicsExposures(p_xv->display, p_xv->gc, True);
    XvSelectVideoNotify(p_xv->display, p_xv->w_main, True);
    for(;;)
    {
#ifndef NOTDEF
        XWindowEvent(p_display, p_xv->w_main, 0xffffffff,  &event);
#else
        XNextEvent(p_display, &event); 
        printf("Event = %d (", event.type);
        print_event(stdout, &event);
        printf(")\n");
#endif
        switch ( event.type )
        {
          case ButtonPress:
            break;
          case KeyPress:
            p_image = read_sun_raster(fd, dump_hdr, p_display, &width, &height);
            if ( p_image )
            {
                XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, 
                p_xv->gc, a_ximage, image_len, p_image, 0, 0, width, height, 0, 0, 
                width, height);
            }
            else
            {
                return 0;
            }
            break;
          case VisibilityNotify:
            XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, 
                a_ximage, image_len, p_image, 0, 0, width, height, 
                0, 0, width, height);
            break;
          case Expose:
            XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, 
                a_ximage, image_len, p_image, 0, 0, width, height, 
                0, 0, width, height);
            break;
          case NoExpose:
            printf("NoExpose notify\n");
            break;
          case GraphicsExpose:
            printf("GraphicsExpose notify\n");
            break;
        }
    }
#else
    process_geometry(p_xv, width, height, p_image);
#endif
}

print_event(fd, event)
    FILE*   fd;
    XEvent* event;
{
    switch( event->type )
    {
      case KeyPress:            fprintf(fd, "KeyPress");            break;
      case MappingNotify:       fprintf(fd, "MappingNotify");       break;
      case KeyRelease:          fprintf(fd, "KeyRelease");          break;
      case ButtonPress:         fprintf(fd, "ButtonPress");         break;
      case ButtonRelease:       fprintf(fd, "ButtonRelease");       break;
      case MotionNotify:        fprintf(fd, "MotionNotify");        break;
      case EnterNotify:         fprintf(fd, "EnterNotify");         break;
      case LeaveNotify:         fprintf(fd, "LeaveNotify");         break;
      case FocusIn:             fprintf(fd, "FocusIn");             break;
      case FocusOut:            fprintf(fd, "FocusOut");            break;
      case KeymapNotify:        fprintf(fd, "KeymapNotify");        break;
      case Expose:              fprintf(fd, "Expose");              break;
      case GraphicsExpose:      fprintf(fd, "GraphicsExpose");      break;
      case NoExpose:            fprintf(fd, "NoExpose");            break;
      case VisibilityNotify:    fprintf(fd, "VisibilityNotify");    break;
      case CreateNotify:        fprintf(fd, "CreateNotify");        break;
      case DestroyNotify:       fprintf(fd, "DestroyNotify");       break;
      case UnmapNotify:         fprintf(fd, "UnmapNotify");         break;
      case MapNotify:           fprintf(fd, "MapNotify");           break;
      case MapRequest:          fprintf(fd, "MapRequest");          break;
      case ReparentNotify:      fprintf(fd, "ReparentNotify");      break;
      case ConfigureNotify:     fprintf(fd, "ConfigureNotify");     break;
      case ConfigureRequest:    fprintf(fd, "ConfigureRequest");    break;
      case GravityNotify:       fprintf(fd, "GravityNotify");       break;
      case ResizeRequest:       fprintf(fd, "ResizeRequest");       break;
      case CirculateNotify:     fprintf(fd, "CirculateNotify");     break;
      case CirculateRequest:    fprintf(fd, "CirculateRequest");    break;
      case PropertyNotify:      fprintf(fd, "PropertyNotify");      break;
      case SelectionClear:      fprintf(fd, "SelectionClear");      break;
      case SelectionRequest:    fprintf(fd, "SelectionRequest");    break;
      case SelectionNotify:     fprintf(fd, "SelectionNotify");     break;
      case ColormapNotify:      fprintf(fd, "ColormapNotify");      break;
      case ClientMessage:       fprintf(fd, "ClientMessage");       break;
    }
}
